(*========================================================================== *)
(*                                                                           *)
(*            (c) Copyright, Ons Seddiki  2013-14                            *)
(*                           Hardware Verification Group,                    *)
(*                           Concordia University                            *)
(*                                                                           *)
(*                Contact:   <o_sed@encs.concordia.ca>                       *)
(*                                                                           *)
(* This file reads the OpenMath XML encoding file and translates it into     *)
(* a HOL Light  output axiom.                                                *)
(* ========================================================================= *)



#directory "+xml-light" ;;
#load "xml-light.cma" ;;
#load "str.cma";;


open String

type lexeme =
  | Nil
  | Numf of float
  | Num of string
  | Str of string
  | Symb of string
  | List of lexeme list

type expr =
  | Nbf of float
  | Nb of string
  | Stri of string
  | Complex of expr * expr
  | Complex1 of expr
  | Vect4 of expr * expr *  expr * expr
  | Vect3 of expr * expr *  expr
  | Vect2 of expr * expr
  | Vect1 of expr
  | Eigenvect  of expr
  | Eigenval  of expr
  | Gt of expr * expr
  | Egt of expr * expr
  | Ls of expr * expr
  | Els of expr * expr
  | Eq of expr * expr
  | Neq of expr * expr
  | Add of expr * expr
  | Sub of expr * expr
  | Mul of expr * expr
  | Div of expr * expr
  | Pow of expr * expr
  | Rule of expr * expr
  | Integrate of expr * expr
  | Diff of expr * expr
  | Partialdiff of expr * expr
  | Neg of int
  | Exp of expr
  | Sin of expr
  | Sqrt of expr
  | Arcsin of expr
  | Cos of expr
  | Arccos of expr
  | Lambda of   expr
  | Tan of expr
  | Arctan of expr
  | Abs of expr
  | Ln of expr
  | Arg of expr
  | Conj of expr
  | Im of expr
  | Re of expr
  | Det of expr
  | Matrix2 of expr *  expr
  | Matrix3 of expr *  expr *  expr
  | Matrix4 of expr *  expr *  expr *  expr
  | Complexmat of expr

let file = "OpenMathoutput.xml"



let rec extract_data acc  = function
  | Xml.Element ("OMA", _,children) ->  List (List.fold_left extract_data [] children) :: acc
  | Xml.Element ("OMBIND", _,children) ->  List (List.fold_left extract_data [] children) :: acc
  | Xml.Element ("OMBVAR", _,children) ->  acc
  | Xml.Element ("OMS", attr, _) ->   let name = List.assoc "name" attr in (match name with
                                                 "pi" -> Str name :: acc
                                               | "e" -> Str name :: acc
                                               | "lambda"  -> acc
                                               | _ -> Symb name :: acc)
  | Xml.Element ("OMV", attr, _) ->  Str (List.assoc "name" attr) :: acc
  | Xml.Element ("OMF", attr, _) ->  Numf (float_of_string (List.assoc "dec" attr)) :: acc
  | Xml.Element ("OMI", _, [child]) ->   Num (Xml.pcdata child) :: acc
  | _ -> acc

let extract file =
  match Xml.parse_file file with
  | Xml.Element ("OMOBJ", _, children) -> List.fold_left extract_data [] children
  | _ -> invalid_arg "Missing XML element 'OMOBJ'."


let rec compareVs v1 v2 = match v1, v2 with
| [], []       -> true
| [], _
| _, []        -> false
  | x::xs, y::ys -> x = y && compareVs xs ys

let rec change_list l =  match l with
[] -> []
| [List l] -> [List (change_list l)]
|[List l1;  List l2; Symb "plus"] -> [List (change_list l1); List (change_list l2); Symb "plus"]
|[List l1;  List l2; Symb "times"] -> [List (change_list l1); List (change_list l2); Symb "times"]
|[List l1;  List l2; Symb "minus"] -> [List (change_list l1); List (change_list l2); Symb "minus"]
|[List l1;  List l2; Symb "divide"] -> [List (change_list l1); List (change_list l2); Symb "divide"]
|[List l1;  List l2; Symb "rational"] -> [List (change_list l1); List (change_list l2); Symb "rational"]
|[List l1;  List l2; Symb "matrixrow"] -> [List (change_list l1); List (change_list l2); Symb "matrixrow"]
|[List l1;  List l2; Symb "matrix"] -> [List (change_list l1); List (change_list l2); Symb "matrix"]


|[List l1;  List l2; Num n; Symb "plus"] -> [List (change_list l1); List (change_list l2); Num ("&"^n); Symb "plus"]
|[List l1;  List l2; Num n; Symb "times"] -> [List (change_list l1); List (change_list l2); Num ("&"^n); Symb "times"]
|[List l1;  List l2; Num n; Symb "minus"] -> [List (change_list l1); List (change_list l2); Num ("&"^n); Symb "minus"]
|[List l1;  List l2; Num n; Symb "divide"] -> [List (change_list l1); List (change_list l2); Num ("&"^n); Symb "divide"]
|[List l1;  List l2;Num n; Symb "rational"] -> [List (change_list l1); List (change_list l2);Num ("&"^n); Symb "rational"]
|[List l1;  List l2; Num n; Symb "matrixrow"] -> [List (change_list l1); List (change_list l2); Num ("&"^n); Symb "matrixrow"]

|[List l1;  List l2; Num n; Symb "matrix"] -> [List (change_list l1); List (change_list l2); Num ("&"^n); Symb "matrix"]




|[List l1;  List l2; List l3; Symb "plus"] -> [List (change_list l1); List (change_list l2); List (change_list l3); Symb "plus"]
|[List l1;  List l2; List l3; Symb "times"] -> [List (change_list l1); List (change_list l2); List (change_list l3); Symb "times"]
|[List l1;  List l2; List l3; Symb "minus"] -> [List (change_list l1); List (change_list l2); List (change_list l3); Symb "minus"]
|[List l1;  List l2; List l3; Symb "divide"] -> [List (change_list l1); List (change_list l2); List (change_list l3); Symb "divide"]
|[List l1;  List l2; List l3; Symb "rational"] -> [List (change_list l1); List (change_list l2); List (change_list l3); Symb "rational"]
|[List l1;  List l2; List l3; Symb "matrixrow"] -> [List (change_list l1); List (change_list l2); List (change_list l3); Symb "matrixrow"]
|[List l1;  List l2; List l3; Symb "matrix"] -> [List (change_list l1); List (change_list l2); List (change_list l3); Symb "matrix"]



|[List l1;  List l2; List l3; List l4; Symb "plus"] -> [List (change_list l1); List (change_list l2); List (change_list l3); List (change_list l4); Symb "plus"]
|[List l1;  List l2; List l3; List l4; Symb "times"] -> [List (change_list l1); List (change_list l2); List (change_list l3); List (change_list l4); Symb "times"]
|[List l1;  List l2; List l3; List l4; Symb "minus"] -> [List (change_list l1); List (change_list l2); List (change_list l3); List (change_list l4); Symb "minus"]
|[List l1;  List l2; List l3; List l4; Symb "divide"] -> [List (change_list l1); List (change_list l2); List (change_list l3); List (change_list l4); Symb "divide"]
|[List l1;  List l2; List l3; List l4; Symb "rational"] -> [List (change_list l1); List (change_list l2); List (change_list l3); List (change_list l4); Symb "rational"]
|[List l1;  List l2; List l3; List l4; Symb "matrixrow"] -> [List (change_list l1); List (change_list l2); List (change_list l3); List (change_list l4); Symb "matrixrow"]
|[List l1;  List l2; List l3; List l4; Symb "matrix"] -> [List (change_list l1); List (change_list l2); List (change_list l3); List (change_list l4); Symb "matrix"]


|[List l; Num n; Symb "power"] -> if (compareVs l ([Num "2"; Num "1"; Symb "rational"]) ) then
                                [ Num ("&"^n); Symb "sqrt"] else [List (change_list l); Num ("&"^n); Symb "power"]
|[List l; Num n; Symb "plus"] -> [List (change_list l); Num ("&"^n); Symb "plus"]
|[List l; Num n; Symb "divide"] -> [List (change_list l); Num ("&"^n); Symb "divide"]
|[List l; Num n; Symb "times"] -> [List (change_list l); Num ("&"^n); Symb "times"]
|[List l; Num n; Symb "minus"] -> [List (change_list l); Num ("&"^n); Symb "minus"]
|[List l; Num n; Symb "rational"] -> [List (change_list l); Num ("&"^n); Symb "rational"]
|[List l; Num n; Symb "Rule"] -> [List (change_list l); Num ("&"^n); Symb "Rule"]


|[Num n;List l; Symb "power"] -> if (n = "-1") then  [List (change_list l); Num "&1";Symb "divide"] else [Num n; List (change_list l);  Symb "power"]
|[Num n;List l; Symb "plus"] -> [Num ("&"^n); List (change_list l);  Symb "plus"]
|[Num n; List l; Symb "divide"] -> [Num ("&"^n); List (change_list l);  Symb "divide"]
|[Num n; List l; Symb "times"] -> [Num ("&"^n); Num ("&"^n); List (change_list l);  Symb "times"]
|[Num n; List l; Symb "minus"] -> [ Num ("&"^n); List (change_list l);  Symb "minus"]
|[Num n; List l; Symb "rational"] -> [  Num ("&"^n);List (change_list l); Symb "rational"]
|[Num n; List l; Symb "Rule"] -> [  Num ("&"^n);List (change_list l); Symb "Rule"]
|[Num n; List l; Symb "matrixrow"] -> [  Num ("&"^n);List (change_list l); Symb "matrixrow"]
|[Num n; List l; Symb "matrix"] -> [  Num ("&"^n);List (change_list l); Symb "matrix"]


|[List l; Str s; Symb "times"] -> [List (change_list l); Str s; Symb "times"]
|[List l; Str s; Symb "plus"] -> [List (change_list l); Str s; Symb "plus"]
|[List l; Str s; Symb "minus"] -> [List (change_list l); Str s; Symb "minus"]
|[List l; Str s; Symb "divide"] -> [List (change_list l); Str s; Symb "divide"]
|[List l; Str s; Symb "rational"] -> [List (change_list l); Str s; Symb "rational"]
|[List l; Str s; Symb "Rule"] -> [List (change_list l); Str s; Symb "Rule"]

|[Str s; List l;  Symb "times"] -> [ Str s; List (change_list l); Symb "times"]
|[Str s; List l;  Symb "plus"] -> [ Str s; List (change_list l); Symb "plus"]
|[Str s; List l;  Symb "minus"] -> [ Str s; List (change_list l); Symb "minus"]
|[Str s; List l;  Symb "divide"] -> [ Str s; List (change_list l); Symb "divide"]
|[Str s; List l;  Symb "rational"] -> [ Str s; List (change_list l); Symb "rational"]
|[Str s; List l;  Symb "Rule"] -> [ Str s; List (change_list l); Symb "Rule"]


| [Num n; Str s; Symb "power"] -> if (n = "-1") then  [Str s ; Num "&1";Symb "divide"] else [Num n; Str s ; Symb "power"]
| [Num n; Str s; Symb "plus"] -> [Num ("&"^n); Str s ; Symb "plus"]
| [Num n; Str s; Symb "minus"] -> [Num ("&"^n); Str s ; Symb "minus"]
| [Num n; Str s; Symb "times"] -> [Num ("&"^n); Str s ; Symb "times"]
| [Num n; Str s; Symb "rational"] -> [Num ("&"^n); Str s ; Symb "rational"]
| [Num n; Str s; Symb "divide"] -> [Num ("&"^n); Str s ; Symb "divide"]
| [Num n; Str s; Symb "matrixrow"] -> [Num ("&"^n); Str s ; Symb "matrixrow"]
| [Num n; Str s; Symb "complex_cartesian"] -> [Num ("&"^n); Str s ; Symb "complex_cartesian"]
| [Num n; Str s; Symb "Rule"] -> [Num ("&"^n); Str s ; Symb "Rule"]

| [Str s;Num n; Symb "power"] -> [Str s ; Num n; Symb "power"]
| [Str s;Num n; Symb "plus"] -> [Str s ; Num ("&"^n); Symb "plus"]
| [Str s; Num n; Symb "minus"] -> [Str s ; Num ("&"^n); Symb "minus"]
| [Str s; Num n; Symb "times"] -> [Str s ; Num ("&"^n); Symb "times"]
| [Str s; Num n; Symb "rational"] -> [Str s ; Num ("&"^n); Symb "rational"]
| [Str s; Num n; Symb "divide"] -> [Str s ; Num ("&"^n); Symb "divide"]
| [Str s; Num n; Symb "matrixrow"] -> [Str s ; Num ("&"^n); Symb "matrixrow"]
| [Str s; Num n; Symb "complex_cartesian"] -> [Str s ; Num ("&"^n); Symb "complex_cartesian"]
| [Str s; Num n; Symb "Rule"] -> [Str s ; Num ("&"^n); Symb "Rule"]

| [Num n; Num m; Symb "power"] -> if (n = "-1") then  [Num ("&"^m) ; Num "&1";Symb "divide"] else [Num n; Num ("&"^m) ; Symb "power"]


| [Num n; Num m; Symb "plus"] -> [Num ("&"^n); Num ("&"^m) ; Symb "plus"]
| [Num n; Num m; Symb "minus"] -> [Num ("&"^n); Num ("&"^m) ; Symb "minus"]
| [Num n; Num m; Symb "times"] -> [Num ("&"^n); Num ("&"^m) ; Symb "times"]
| [Num n; Num m; Symb "rational"] -> [Num ("&"^n); Num ("&"^m) ; Symb "rational"]
| [Num n; Num m; Symb "divide"] -> [Num ("&"^n);Num ("&"^m) ; Symb "divide"]
| [Num n; Num m; Symb "matrixrow"] -> [Num ("&"^n);Num ("&"^m) ; Symb "matrixrow"]
| [Num n; Num m; Symb "complex_cartesian"] -> [Num ("&"^n);Num ("&"^m) ; Symb "complex_cartesian"]
| [Num n; Num m; Symb "Rule"] -> [Num ("&"^n);Num ("&"^m) ; Symb "Rule"]


(*| [List[Num n; Num m;Num p; Num q;  Symb "matrixrow"]] -> [List[Num ("&"^n);Num ("&"^m) ; Num ("&"^p);Num ("&"^q) ; Symb "matrixrow"]]*)
|[Num n; Num m;Num p; Num q;  Symb "matrixrow"] -> [Num ("&"^n);Num ("&"^m) ; Num ("&"^p);Num ("&"^q) ; Symb "matrixrow"]
(*|[List [Num n; Num m;Num p;   Symb "matrixrow"]] -> [List [Num ("&"^n);Num ("&"^m) ; Num ("&"^p); Symb "matrixrow"]]*)
|[Num n; Num m;Num p;   Symb "matrixrow"] ->  [Num ("&"^n);Num ("&"^m) ; Num ("&"^p); Symb "matrixrow"]


| [List l;  Symb "matrixrow"] -> [List (change_list l); Symb "matrixrow"]



| [List l; List l1;Str p; Symb "matrixrow"] -> [List (change_list l);List (change_list l1) ; Str p; Symb "matrixrow"]

| [List l; Num p; List l1; Symb "matrixrow"] -> [List (change_list l);Num ("&"^p); List (change_list l1) ; Symb "matrixrow"]
| [List l; Str p; List l1; Symb "matrixrow"] -> [List (change_list l);Str p; List (change_list l1) ; Symb "matrixrow"]

| [Str p; List l; List l1; Symb "matrixrow"] -> [Str p; List (change_list l); List (change_list l1) ; Symb "matrixrow"]
| [Num p; List l; List l1; Symb "matrixrow"] -> [Num ("&"^p); List (change_list l); List (change_list l1) ; Symb "matrixrow"]




| [List [Num "2"; Num "1"; Symb "rational"]; Str n; Symb "power"]-> [Str n; Symb "sqrt"]
| [List [Num "2"; Num "1"; Symb "rational"]; List l; Symb "power"]-> [List (change_list l);  Symb "sqrt"]





|[List [Str x; List [Num "1"; Num "0"; Symb "complex_cartesian"]; Symb "times"] ;Str "e"; Symb "power"]-> [List [Str "ii";Str x; Symb "times"]; Symb "exp"]


| _ -> l




let trim str =   if str = "" then "" else   let search_pos init p next =
    let rec search i =
      if p i then raise(Failure "empty") else
      match str.[i] with
      | ' ' | '\n' | '\r' | '\t'| '-' | '&' -> search (next i)
      | _ -> i
    in
    search init   in   let len = String.length str in   try
    let left = search_pos 0 (fun i -> i >= len) (succ)
    and right = search_pos (len - 1) (fun i -> i < 0) (pred)
    in
    String.sub str left (right - left + 1)   with   | Failure "empty" -> ""





let prior c = if (c = "plus") or (c = "subtract") then 2 else if (c = "gt") or (c = "lt") or (c="eq") or (c="leq") or (c="geq") or (c="neq") then 1 else 3
let rec priority  = function
  |[] | [_] -> 4
(*  |[a;b] -> failwith "Output misused: File parserand collector"*)
  |t :: (Symb v):: q -> if ((priority q) = 4) then (prior v) else  (max (prior v) (priority q))
|_ -> failwith "Output misused: File parserand collector"

let rec syntax liste = match liste with
  | [] | [Symb _] -> failwith "Invalid Output: File parserand collector"
  | [Num n] -> Nb n
  | [Numf f] -> Nbf f
  | [Str s] -> Stri s
  | [List l] -> syntax (List.rev l)
  | [Symb "-"; Num n] -> Neg (int_of_string n)
  | [Symb "+"; n] -> syntax [n]
  | [Symb "cos"; n] -> Cos (syntax [n])
  | [Symb "sin"; n] -> Sin (syntax [n])
  | [Symb "abs"; n] -> Abs (syntax [n])
  | [Symb "tan"; n] -> Tan (syntax [n])
  | [Symb "sqrt"; n] -> Sqrt (syntax [n])
  | [Symb "arcsin"; n] -> Arcsin (syntax [n])
  | [Symb "arccos"; n] -> Arccos (syntax [n])
  | [Symb "arctan"; n] -> Arctan (syntax [n])
  | [Symb "exp"; n] -> Exp (syntax [n])
 (* | [Symb "E"; n] -> Exp (syntax [n])*)
  | [Symb "ln"; n] -> Ln (syntax [n])
  | [Symb "argument"; n] -> Arg (syntax [n])
  | [Symb "conjugate"; n] -> Conj (syntax [n])
  | [Symb "imaginary"; n] -> Im (syntax [n])
  | [Symb "real"; n] -> Re (syntax [n])
  | [Symb "det"; n] -> Det (syntax [n])
  | [Symb "complex_cartesian"; a;b] -> Complex (syntax [a],syntax [b])
  |[Symb "Rule"; a ; b] -> Rule (syntax [a],syntax [b])
(*  | [Symb "Rule";  List a ; b] -> Rule (syntax (List.rev a),syntax [b])
  | [Symb "Rule";  b;  List a ] -> Rule (syntax [b],syntax (List.rev a))*)

  | [Symb "eigenvect"; n] -> Eigenvect (syntax [n])
  | [Symb "eigenvalues"; n] -> Eigenval (syntax [n])
  | [Symb "integrate"; List [a;b]] -> Integrate (syntax [a],syntax [b])
  | [Symb "diff"; List [a;b]] -> Diff (syntax [a],syntax [b])
  | [Symb "partialdiff"; List [a;b]] -> Partialdiff (syntax [a],syntax [b])
  | [Symb "matrix"; a;b] -> Matrix2 (syntax [a],syntax [b])
  | [Symb "matrix"; a;b;c] -> Matrix3 (syntax [a],syntax [b],syntax [c])
  | [Symb "matrix"; a;b;c;d] -> Matrix4 (syntax [a],syntax [b],syntax [c],syntax [d])


  | [Symb "matrixrow"; a;b;c;d] -> Vect4 (syntax [a],syntax [b],syntax [c],syntax [d])
  | [Symb "matrixrow"; a;b;c] -> Vect3 (syntax [a],syntax [b],syntax [c])
 (* | [Symb "matrixrow"; d;List a;List a1] -> Vect3 (syntax [d],syntax (List.rev a),syntax (List.rev a1))*)
  | [Symb "matrixrow"; a] -> Vect1 (syntax [a])
 | [Symb "matrixrow";a;b] -> Vect2 (syntax [a],syntax [b])
 (*| [Symb "matrixrow";List a] -> Vect1 (syntax (List.rev a))*)
  | [ Symb c; before; after] |  [ before; after; Symb c] | [ before; Symb c; after] -> if (prior c) < priority [after] then (match c with
                                | "plus" -> Add ((syntax [before]),(syntax [after]))
                                | "subtract" -> Sub ((syntax [before]),(syntax [after]))
                                | "times" -> Mul ((syntax [before]),(syntax [after]))
                                | "divide" -> Div ((syntax [before]),(syntax [after]))
                                | "rational" -> Div ((syntax [before]),(syntax [after]))
                                | "gt" -> Gt ((syntax [before]),(syntax [after]))
                                                                | "leq" -> Els ((syntax [before]),(syntax [after]))
                                                                | "geq" -> Egt ((syntax [before]),(syntax [after]))
                                                                | "neq" -> Neq ((syntax [before]),(syntax [after]))
                                                                | "eq" -> Eq ((syntax [before]),(syntax [after]))
                                                                | "lt" -> Ls ((syntax [before]),(syntax [after]))
                                | "power" -> Pow ((syntax [before]),(syntax [after]))
                                | _ -> failwith "Invalid Output: File parserand collector")
                              else syntax ((List [before; List.hd [after]; Symb c;])::(List.tl [after]));

  | _ -> failwith "File parserandcollector: The Output expression misused: missing brackets"

let rec splitlex l lex = match l with
x :: after -> if x = lex then [[];after] else  [List.flatten [x:: List.hd (splitlex after lex)]; List.flatten (List.tl (splitlex after lex)) ]
|[] -> failwith "File parserandsplitter: Invalid splitlex liste"


(*let rec atom liste = match liste with
| [List l] -> atom l
| _->  if (List.mem (Symb "=") liste) then   (match "=" with
                                |   "=" -> Eq ((syntax (List.hd (splitlex liste (Symb "=")))),(syntax(List.hd(List.tl(splitlex liste (Symb "="))))))
                                | _ -> failwith "File parserandsplitter: The Invalid Input: \"=\" Operator")
       else if (List.mem (Symb "!=") liste) then   (match "!=" with
                                |  "!=" -> Neq ((syntax (List.hd (splitlex liste (Symb "!=")))),(syntax(List.hd(List.tl(splitlex liste (Symb "!="))))))
                                | _ -> failwith "File parserandsplitter: The Invalid Input: \"!=\" Operator")
      else if (List.mem (Symb "<=") liste) then   (match "<=" with
                                | "<=" -> Els ((syntax (List.hd (splitlex liste (Symb "<=")))),(syntax(List.hd(List.tl(splitlex liste (Symb "<="))))))
                                | _ -> failwith "File parserandsplitter: The Invalid Input: \"<=\" Operator")
     else if (List.mem (Symb "<") liste) then   (match "<" with
                                |  "<" -> Ls ((syntax (List.hd (splitlex liste (Symb "<")))),(syntax(List.hd(List.tl(splitlex liste (Symb "<"))))))
                                | _ -> failwith "File parserandsplitter: The Invalid Input: \"<\" Operator")
    else if (List.mem (Symb ">=") liste) then   (match ">=" with
                                |  ">=" -> Egt ((syntax (List.hd (splitlex liste (Symb ">=")))),(syntax(List.hd(List.tl(splitlex liste (Symb ">="))))))
                                | _ -> failwith "File parserandsplitter: The Invalid Input: \">=\" Operator")
  else if (List.mem (Symb ">") liste) then   (match ">" with
                                |  ">" -> Gt ((syntax (List.hd (splitlex liste (Symb ">")))),(syntax(List.hd(List.tl(splitlex liste (Symb ">"))))))
                                | _ -> failwith "File parserandsplitter: The Invalid Input: \">\" Operator")
 else if (List.mem (Symb "and") liste) then   (match "and" with
                                |  "and" -> And ((atom (List.hd (splitlex liste (Symb "and")))),(atom(List.hd(List.tl(splitlex liste (Symb "and"))))))
                                | _ -> failwith "File parserandsplitter: The Invalid Input: \"and\" Operator")

else failwith "File parserandsplitter: The input expression misused: missing brackets"*)





let string_of_char = String.make 1


let rec calc expr = match expr with
  | Nb s ->  if String.contains s '-' then  "(-- &"^ (trim s) ^")"   else  "&"^s
  | Nbf f ->   "#"^string_of_float  f  ^ " "
  | Stri s ->  s

  | Add(a,b) -> "("^ calc a ^" + "^ calc b   ^ ")"
  | Sub(a,b) -> "("^ calc a ^" - "^ calc b   ^ ")"
  | Mul(a,b) -> "("^calc a ^" * "^ calc b   ^ ")"
  | Div(a,b) -> "("^calc a ^" / "^ calc b   ^ ")"
  | Gt(a,b) -> calc a ^">"^ calc b
  | Egt(a,b) -> calc a ^">="^ calc b
  | Ls(a,b) -> calc a ^"<"^ calc b
  | Els(a,b) -> calc a ^"<="^ calc b
  | Eq(a,b) -> calc a ^"="^ calc b
  | Neq(a,b) ->calc a ^"!="^ calc b
  | Neg a ->  "-- &" ^string_of_int a
  | Sin a -> "sin( " ^ calc a ^")"
  | Arcsin a -> "arcsin( " ^ calc a ^")"
  | Cos a -> "cos( " ^ calc a ^")"
  | Sqrt a -> "sqrt( " ^ calc a ^")"
  | Arccos a -> "arcos( " ^ calc a ^")"
  | Tan a -> "tan( " ^ calc a ^")"
  | Arctan a -> "arctan( " ^ calc a ^")"
  | Pow (a,b) -> "("^calc a ^" rpow("^ calc b  ^ "))"
  | Rule (a,b) -> "("^calc a ^" = "^ calc b  ^ ")"
  | Abs a ->"abs( " ^ calc a ^")"
  | Exp a ->"exp( " ^ calc a ^")"
  | Ln a ->"ln( " ^ calc a ^")"
  | Vect4 (a,b,c,d)->"vector[" ^"(" ^ calc a^ ")" ^";"^ "(" ^ calc b^ ")"^ ";"^"(" ^ calc c ^")"^ ";"^"(" ^ calc d ^")" ^"]"
  | Vect3 (a,b,c)->"vector[" ^ "("^calc a^")" ^";"^ "("^calc b ^")"^ ";"^ "("^calc c^")" ^ "]"
  | Vect2 (a,b)->"vector[" ^"("^ calc a^")" ^ ";" ^"("^ calc b^")"   ^"]"
  | Vect1 (a)-> calc a
  | Complex(a,b) ->"complex" ^ "(" ^ calc a ^"  ,  "^ calc b ^")"
  | Complex1(a) ->"complex" ^ "(" ^ calc a  ^")"
  | Eigenval(a) ->"eigen_val"^ calc a
  | Eigenvect(a) ->"eigen_vect"^ calc a

  | Matrix2(a,b) -> "vector["^"("^calc a^")"  ^ ";" ^ "("^calc b^")"^"]"
  | Matrix3(a,b,c) -> "vector["^"("^calc a^")"  ^ ";" ^"("^ calc b^")"^ ";" ^"("^ calc c^")"^"]"
  | Matrix4(a,b,c,d) -> "vector["^"("^calc a^")"  ^ ";" ^"("^ calc b^")"^ ";" ^ "("^calc c^")"^ ";" ^ "("^calc d^")"^"]"
  | Arg a -> "arg( " ^ calc a  ^")"
  | Conj a -> "conj( " ^ calc a  ^")"
  | Re a -> "Re( "^ calc a   ^")"
  | Im a -> "Im( "^ calc a   ^")"
  | Det a -> "det( " ^ calc a  ^")"
  | Integrate(a,b) -> "int ("^calc a^")"
  | Diff(a,b) -> "diff ("^calc a^")"
  | Partialdiff(a,b) -> "partialdiff ("^calc a^")"
  | Complexmat a -> calc a ^ " "
  |_ -> failwith "File parserandcollector: The HOL Light expression does not exist"
  
  
  
let rec splitchar s c =
    if String.contains s c then
        (String.sub s 0 ((String.index s c))) :: (splitchar (String.sub s ((String.index s c)+ 1) ((String.length s) - (String.index s c) - 1)) c)
    else [s]
  
  
  
  
  
  let contains s1 s2 =
    let re = Str.regexp_string s2
    in
        try ignore (Str.search_forward re s1 0); true
              with Not_found -> false




let change_vect s=
let s1= String.sub s 0 6 in
if (s1 ="vector") then
let str = String.sub s 7 ((String.length s) - 8) in
let l = splitchar str ';' in
String.concat " " l
else 
s

let change_vect3 s=
let subb = String.sub s 7 ((String.length s) - 8) in
let sub = String.sub subb 1 ((String.length subb) - 1) in
let indbc =  (String.index sub ']')   in
let indpv =  (String.index_from sub indbc ';')  in
let sub1 = String.sub sub 0 ( indbc + 1) in
let indbc1 =  (String.index_from sub (indbc + 1) ']') in
let indpv1 =  (String.index_from sub indbc1 ';')  in
let sub2 = String.sub sub (indpv + 2)  ( indbc1 - indpv - 1  ) in
let sub3 = String.sub sub (indpv1 + 2)  ( (String.length sub) - indpv1 - 3) in
 "mat3x3 " ^ change_vect sub1 ^ " " ^ change_vect sub2 ^ " " ^ change_vect sub3

 let change_vect4 s=
let subb = String.sub s 7 ((String.length s) - 8) in
let sub = String.sub subb 1 ((String.length subb) - 1) in
let indbc =  (String.index sub ']')   in
let indpv =  (String.index_from sub indbc ';')  in
let sub1 = String.sub sub 0 ( indbc + 1 ) in
let indbc1 =  (String.index_from sub (indbc + 1) ']') in
let indpv1 =  (String.index_from sub indbc1 ';')  in
let sub2 = String.sub sub (indpv + 2)  ( indbc1 - indpv - 1  ) in
let indbc2 =  (String.index_from sub (indbc1 + 1) ']') in
let indpv2 =  (String.index_from sub indbc2 ';')  in
let sub3 = String.sub sub (indpv1 + 2)  ( indbc2 - indpv1 - 1  ) in
let sub4 = String.sub sub (indpv2 + 2)  ( (String.length sub) - indpv2 - 3) in
 "mat4x4 " ^ change_vect sub1 ^ " " ^ change_vect sub2 ^ " " ^ change_vect sub3^ " " ^ change_vect sub4
 

let change_matrix res =
let subb = String.sub res 7 ((String.length res) - 8) in
let sub = String.sub subb 1 ((String.length subb) - 1) in
let l = splitchar sub ';'  in
if (List.length l = 4) then
begin
let indbc =  (String.index sub ']')   in
let sub1 = String.sub sub 0 (indbc + 1) in
let indpv =  (String.index_from sub indbc ';') + 1 in
let sub2 = String.sub sub (indpv + 1)  ((String.length  sub) - indpv - 2 ) in
"mat2x2" ^ " " ^ change_vect sub1 ^ " " ^ change_vect sub2
end
else if (List.length l = 9) then
begin
 change_vect3 res
end
else if (List.length l = 16) then
begin
 change_vect4 res
end
else subb
    


  
  
let call_collector1()  =
calc(syntax(extract file)) 


let call_collector()  =
let res = calc(syntax(extract file)) in
if (contains res "vector") then
begin
let subb = String.sub res 7 ((String.length res) - 8) in
if (contains subb "vector") then
change_matrix res
else 
res
end
else
res
