/*
 * $Id: MathematicaCodec.java,v 1.7 2002/11/20 23:51:51 mriem Exp $
 *
 * Copyright (c) 1999-2002, RIACA, Technische Universiteit Eindhoven (TU/e).
 * All Rights Reserved.
 *
 * The contents of this file are subject to the RIACA Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.riaca.win.tue.nl/
 */

package mathematicaCodec;
import nl.tue.win.riaca.openmath.codec.*;
import nl.tue.win.riaca.openmath.lang.*;
import nl.tue.win.riaca.openmath.io.*;
import org.xml.sax.*;
import java.util.*;
import java.net.*;
import java.io.*;

/**
 * A Codec that translates (XML-encoded) OpenMath objects to their Mathematica 
 * equivalent and vice versa. <p>
 *
 * @author Manfred Riem (mriem@win.tue.nl)
 * @version 1.1
 */

public class MathematicaCodec
    extends Codec
{
    /**
     * Stores the hashtable for CD matching. <p>
     *
     * This hashtable is used to lookup which codecs handles
     * a given Mathematica full form operator. <p>
     *
     * Eg. Plus[ 1, 2 ] -> arith1 (so the arith1Codec). <p>
     *
     * @since 1.0
     */
    
    protected Hashtable mDecodeHashtable = new Hashtable();
    
    
    /**
     * Constructor. <p>
     *
     * Note that we load the codec here with the entire OpenMath core
     * set of CDs (and there associated Mathematica symbols). If one
     * of the core codecs fails, we silently continue. <p>
     *
     * @since 1.0
     */
    
    public MathematicaCodec()
    {
	// System.out.println( "MathematicaCodec.<init>" );
	
        /*
         * Try to add the 'core' CD's. Silently fail if it did not succeed.
         */
	
        try
        {
	    addCD( "arith1",    "", 
		   "mathematicaCodec.arith1Codec"    );
    	    addCD( "calculus1", "", 
		   "mathematicaCodec.calculus1Codec" );
    	    addCD( "complex1",  "", 
		   "mathematicaCodec.complex1Codec"  );
    	    addCD( "fns1",      "", 
		   "mathematicaCodec.fns1Codec"      );
    	    addCD( "logic1",  "", 
		   "mathematicaCodec.logic1Codec"  );
    	    addCD( "integer1",  "", 
    	 		   "mathematicaCodec.integer1Codec"  );
    	   // addCD( "list1",     "", 
		    //"mathematicaCodec.list1Codec"     );
    	    addCD( "linalg1",   "", 
		   "mathematicaCodec.linalg1Codec"   );
    	    addCD( "nums1",     "", 
		   "mathematicaCodec.nums1Codec"     );
    	    addCD( "quant1",    "", 
		   "mathematicaCodec.quant1Codec"    );
    	    addCD( "relation1", "", 
		   "mathematicaCodec.relation1Codec" );
    	    addCD( "transc1",   "", 
		   "mathematicaCodec.transc1Codec"   );
    	    addCD( "linalg2",   "", 
    	 	"mathematicaCodec.linalg2Codec"   );
    	    addCD( "linalgeig1",   "", 
    	    "mathematicaCodec.linalgeig1Codec"   );
    	    addCD( "linalg4",   "", 
    	 	"mathematicaCodec.linalg4Codec"   );
    	    
    	}
    	catch( CodecException tCodecException )
    	{
    	    tCodecException.printStackTrace();
    	}
        
	mDecodeHashtable.put( "Abs",           "arith1" );
	mDecodeHashtable.put( "Divide",        "arith1" );
	mDecodeHashtable.put( "GCD",           "arith1" );
	mDecodeHashtable.put( "LCM",           "arith1" );
	mDecodeHashtable.put( "Subtract",      "arith1" );
	mDecodeHashtable.put( "Plus",          "arith1" );
	mDecodeHashtable.put( "Power",         "arith1" );
	mDecodeHashtable.put( "Product",       "arith1" );
	mDecodeHashtable.put( "Sqrt",          "arith1" );
	mDecodeHashtable.put( "Sum",           "arith1" );
	mDecodeHashtable.put( "Times",         "arith1" );
	mDecodeHashtable.put( "Minus",         "arith1" );
        
	mDecodeHashtable.put( "Integrate",     "calculus1" );
	mDecodeHashtable.put( "Derivative",    "calculus1" );
	mDecodeHashtable.put( "D",             "calculus1" );
        
	mDecodeHashtable.put( "Arg",           "complex1" );
	mDecodeHashtable.put( "Complex",       "complex1" );
	mDecodeHashtable.put( "Conjugate",     "complex1" );
	mDecodeHashtable.put( "Im",            "complex1" );
	mDecodeHashtable.put( "Re",            "complex1" );
        
	mDecodeHashtable.put( "Function",      "fns1" );
        
        mDecodeHashtable.put( "Factorial",     "integer1" );
        mDecodeHashtable.put( "Quotient",      "integer1" );
        mDecodeHashtable.put( "Mod",           "integer1" );
        
      // mDecodeHashtable.put( "List",          "list1" );
        
        mDecodeHashtable.put( "Det",           "linalg1" );
        mDecodeHashtable.put( "Cross",           "linalg1" );
        mDecodeHashtable.put( "Outer",           "linalg1" );

        
        mDecodeHashtable.put( "E",             "nums1"     );
        mDecodeHashtable.put( "EulerGamma",    "nums1"     );
        mDecodeHashtable.put( "I",             "nums1"     );
        mDecodeHashtable.put( "Infinity",      "nums1"     );
        mDecodeHashtable.put( "Indeterminate", "nums1"     );
        mDecodeHashtable.put( "Pi",            "nums1"     );
        mDecodeHashtable.put( "Rational",      "nums1"     );
        
        mDecodeHashtable.put( "Exists",        "quant1"    );
        mDecodeHashtable.put( "ForAll",        "quant1"    );
        
	mDecodeHashtable.put( "Equal",         "relation1" );
	mDecodeHashtable.put( "GreaterEqual",  "relation1" );
	mDecodeHashtable.put( "Greater",       "relation1" );
	mDecodeHashtable.put( "LessEqual",     "relation1" );
	mDecodeHashtable.put( "Less",          "relation1" );
	mDecodeHashtable.put( "Unequal",       "relation1" );
        
	mDecodeHashtable.put( "ArcCos",        "transc1" );
	mDecodeHashtable.put( "ArcCosh",       "transc1" );
	mDecodeHashtable.put( "ArcCot",        "transc1" );
	mDecodeHashtable.put( "ArcCoth",       "transc1" );
	mDecodeHashtable.put( "ArcCsc",        "transc1" );
	mDecodeHashtable.put( "ArcCsch",       "transc1" );
	mDecodeHashtable.put( "ArcSec",        "transc1" );
	mDecodeHashtable.put( "ArcSech",       "transc1" );
	mDecodeHashtable.put( "ArcSin",        "transc1" );
	mDecodeHashtable.put( "ArcSinh",       "transc1" );
	mDecodeHashtable.put( "ArcTan",        "transc1" );
	mDecodeHashtable.put( "ArcTanh",       "transc1" );
      mDecodeHashtable.put( "Cos",           "transc1" );
        mDecodeHashtable.put( "Cosh",          "transc1" );
        mDecodeHashtable.put( "Cot",           "transc1" );
        mDecodeHashtable.put( "Coth",          "transc1" );
        mDecodeHashtable.put( "Csc",           "transc1" );
        mDecodeHashtable.put( "Csch",          "transc1" );
        mDecodeHashtable.put( "Exp",           "transc1" );
        mDecodeHashtable.put( "Log",           "transc1" );
        mDecodeHashtable.put( "Sec",           "transc1" );
        mDecodeHashtable.put( "Sech",          "transc1" );
        mDecodeHashtable.put( "Sin",           "transc1" );
        mDecodeHashtable.put( "Sinh",          "transc1" );
        mDecodeHashtable.put( "Tan",           "transc1" );
        mDecodeHashtable.put( "Tanh",          "transc1" );
        
        
        mDecodeHashtable.put( "MatrixForm",          "linalg2" );
        mDecodeHashtable.put( "{",          "linalg2" );
        mDecodeHashtable.put( "List",          "linalg2" );
        
        mDecodeHashtable.put( "Eigenvalues",    "linalgeig1" );
        
        mDecodeHashtable.put( "Eigenvectors",          "linalg4" );

        
        
        
        
    }
    
    
    /* -----------------------------------------------------------------------
     *
     *   Encode methods
     *
     * -----------------------------------------------------------------------
     */


    /**
     * Encodes the XML-encoded OM-object to Mathematica-syntax. <p>
     *
     * @param fXMLOMObject the encoded OM-object.
     * @return the string in Mathematica-syntax, 
     *         or <b>null</b> if not successfull.
     * @since 1.0
     */
    
    public String encode( String fXMLOMObject )
        throws CodecEncodeException
    {
	// System.out.println( "MathematicaCodec.encode" );
            
	OMObject tOMObject  = null;
        
	StringBufferInputStream tInputStream = 
            new StringBufferInputStream( fXMLOMObject );
        
																								InputSource tSource = new InputSource( tInputStream );
        OMXMLReader tReader = new OMXMLReader( tSource );
	
        try
        {
            tOMObject = tReader.readObject();
            return encodeOMObject( tOMObject );
        }
        catch( IOException tIOException )
        {
            throw new CodecEncodeException(
	        "Unable to encode: " + fXMLOMObject );
        }
    }
    
    
    /**
     * Encode an OMObject. <p>
     *
     * @throws CodecEncodeException to signal a major failure.
     * @since 1.0
     */
    
    public String encodeOMObject( OMObject fObject ) 
	throws CodecEncodeException
    {
    	// System.out.println( "MathematicaCodec.encodeOMObject" );
        
        if ( fObject instanceof OMInteger     ) 
	    return encodeOMInteger    ( fObject );
        if ( fObject instanceof OMVariable    ) 
	    return encodeOMVariable   ( fObject );
        if ( fObject instanceof OMString      ) 
	    return encodeOMString     ( fObject );
        if ( fObject instanceof OMFloat       ) 
	    return encodeOMFloat      ( fObject );
        if ( fObject instanceof OMApplication ) 
	    return encodeOMApplication( fObject );
        if ( fObject instanceof OMSymbol      ) 
	    return encodeOMSymbol     ( fObject );
        if ( fObject instanceof OMBinding     ) 
	    return encodeOMBinding    ( fObject );
        
        throw new CodecEncodeException( 
            "Unable to encode: " + fObject.toString() );
    }
    
    
    /**
     * Encode an OMInteger. <p>
     *
     * @throws CodecEncodeException to signal a major failure.
     * @since 1.0
     */
    
    protected String encodeOMInteger( OMObject fObject ) 
	throws CodecEncodeException
    {
	// System.out.println( "MathematicaCodec.encodeOMInteger" );
        
        OMInteger tInteger = (OMInteger) fObject;
        return tInteger.getInteger();
    }
    
    
    /**
     * Encode an OMVariable. <p>
     *
     * @since 1.0
     */
    
    protected String encodeOMVariable( OMObject fObject ) 
	throws CodecEncodeException
    {
	// System.out.println( "MathematicaCodec.encodeOMVariable" );
        
        OMVariable tVariable = (OMVariable) fObject;
	return tVariable.getName();
    }
    
    
    /**
     * Encode an OMString. <p>
     *
     * @since 1.0
     */
    
    protected String encodeOMString( OMObject fObject ) 
	throws CodecEncodeException
    {
	// System.out.println( "MathematicaCodec.encodeOMString" );
        
        OMString tString = (OMString) fObject;
	return "\"" + tString.getString() + "\"";
    }
    
    
    /**
     * Encode an OMFloat. <p>
     *
     * @since 1.0
     */
    
    protected String encodeOMFloat( OMObject fObject ) 
	throws CodecEncodeException
    {
	// System.out.println( "MathematicaCodec.encodeOMFloat" );
        
        OMFloat tOMFloat = (OMFloat) fObject;
	return tOMFloat.getFloat();
    }
    
    
    /**
     * Encode an OMApplication. <p>
     *
     * @since 1.0
     */
    
    protected String encodeOMApplication( OMObject fObject ) 
	throws CodecEncodeException
    {
	// System.out.println( "MathematicaCodec.encodeOMApplication" );
        
        if ( fObject instanceof OMApplication )
	{
	    OMApplication tApplication = (OMApplication) fObject;
	    OMObject      tObject      = tApplication.firstElement();
            
	    if ( tObject instanceof OMSymbol )
	    {
		OMSymbol tSymbol = (OMSymbol) tObject;
                
                String tClassName     = "";
                String tClassLocation = "";
                
                if ( mCDs.get( tSymbol.getCD() ) != null )
                {
                    String tLocationClass = (String) 
                        mCDs.get( tSymbol.getCD() );
                    
                    tClassName = tLocationClass.substring( 
                        tLocationClass.lastIndexOf( ":" ) + 1 );
                    
                    tClassLocation = tLocationClass.substring(
                        0, tLocationClass.lastIndexOf( ":" ) );
                }
                else
		    tClassName = 
			"nl.tue.win.riaca.mathematica.codec.core." +
			tSymbol.getCD() + 
			"Codec";
                
                try
                {
                    if ( tClassLocation.equals( "" ) )
                    {
                        Class   tClass    = Class.forName( tClassName );
			Object  tCDObject = tClass.newInstance();
                        CDCodec tCDCodec  = (CDCodec) tCDObject;
                        
                        tCDCodec.setParent( this );
                        return tCDCodec.encodeOMObject( tApplication );
                    }
                    else
                    {
                        URLClassLoader tClassLoader = 
                            new URLClassLoader( 
                                new URL[] { new URL( tClassLocation ) } , 
				this.getClass().getClassLoader() );
			
                        Class   tClass    = 
			    tClassLoader.loadClass( tClassName );
                    	Object  tCDObject = tClass.newInstance();
                        CDCodec tCDCodec  = (CDCodec) tCDObject;
                        
                        tCDCodec.setParent( this );
                        return tCDCodec.encodeOMObject( tApplication );
                    }
		}
                catch( MalformedURLException tMalformedURLException )
                {
                    throw new CodecEncodeException(
                        "MalformedURLException for codec URL: " + 
                        tClassLocation );
                }
                catch( InstantiationException tInstantiationException )
                {
                    throw new CodecEncodeException( 
                        "InstantiationException for codec class: " + 
			tClassName );
                }
                catch( IllegalAccessException tIllegalAccessException )
                {
                    throw new CodecEncodeException(
                        "IllegalAccessException for codec class: " + 
			tClassName );
                }
                catch( ClassNotFoundException tClassNotFoundException )
                {
                    throw new CodecEncodeException(
                        "ClassNotFoundException for codec class: " + 
			tClassName );
                }
	    }
	}
        
        throw new CodecEncodeException( 
	    "Unable to encode OMObject: " + fObject.toString() );
    }
    
    
    /**
     * Encode an OMBinding. <p>
     *
     * @since 1.0
     */
    
    protected String encodeOMBinding( OMObject fOMObject )
        throws CodecEncodeException
    {
        // System.out.println( "MathematicaCodec.encodeOMBinding" );
	
        if ( fOMObject instanceof OMBinding )
	{
	    OMBinding tBinding = (OMBinding) fOMObject;
	    OMObject  tObject  = tBinding.getBinder();
            
	    if ( tObject instanceof OMSymbol )
	    {
		OMSymbol tSymbol = (OMSymbol) tObject;
		
                String tClassName     = "";
                String tClassLocation = "";
                
                if ( mCDs.get( tSymbol.getCD() ) != null )
                {
                    String tLocationClass = (String) 
                        mCDs.get( tSymbol.getCD() );
                    
                    tClassName = tLocationClass.substring( 
                        tLocationClass.lastIndexOf( ":" ) + 1 );
                    
                    tClassLocation = tLocationClass.substring(
                        0, tLocationClass.lastIndexOf( ":" ) );
                }
                else
		    tClassName = 
			"nl.tue.win.riaca.mathematica.codec.core." +
			tSymbol.getCD() +
			"Codec";
                
                try
                {
                    if ( tClassLocation.equals( "" ) )
                    {
                        Class   tClass    = Class.forName( tClassName );
			Object  tCDObject = tClass.newInstance();
                        CDCodec tCDCodec  = (CDCodec) tCDObject;
                        
                        tCDCodec.setParent( this );
                        return tCDCodec.encodeOMObject( tBinding );
                    }
                    else
                    {
                        URLClassLoader tClassLoader = 
                            new URLClassLoader( 
                                new URL[] { new URL( tClassLocation ) } , 
				this.getClass().getClassLoader() );
			
                        Class   tClass    = 
			    tClassLoader.loadClass( tClassName );
			Object  tCDObject = tClass.newInstance();
                        CDCodec tCDCodec  = (CDCodec) tCDObject;
                        
                        tCDCodec.setParent( this );
                        return tCDCodec.encodeOMObject( tBinding );
                    }
		}
                catch( MalformedURLException tMalformedURLException )
                {
                    throw new CodecEncodeException(
                        "MalformedURLException for codec URL: " + 
                        tClassLocation );
                }
                catch( InstantiationException tInstantiationException )
                {
                    throw new CodecEncodeException( 
                        "InstantiationException for codec class: " + 
			tClassName );
                }
                catch( IllegalAccessException tIllegalAccessException )
                {
                    throw new CodecEncodeException(
                        "IllegalAccessException for codec class: " + 
			tClassName );
                }
                catch( ClassNotFoundException tClassNotFoundException )
                {
                    throw new CodecEncodeException(
                        "ClassNotFoundException for codec class: " + 
			tClassName );
                }
	    }
	}
	
        throw new CodecEncodeException( 
            "Unable to encode OMObject: " + fOMObject.toString() );
    }
    
    
    /**
     * Encodes the OM-Symbol to a Mathematica-command. <p>
     *
     * @since 1.0
     */
    
    public String encodeOMSymbol( OMObject fOMObject )
        throws CodecEncodeException
    {
    	// System.out.println( "MathematicaCodec.encodeOMSymbol" );
        
	if ( fOMObject instanceof OMSymbol )
	{
	    OMSymbol tSymbol        = (OMSymbol) fOMObject;
            String   tClassName     = "";
            String   tClassLocation = "";
                
            if ( mCDs.get( tSymbol.getCD() ) != null )
            {
                String tLocationClass = (String) 
                    mCDs.get( tSymbol.getCD() );
                    
                tClassName = tLocationClass.substring( 
                    tLocationClass.lastIndexOf( ":" ) + 1 );
                    
                tClassLocation = tLocationClass.substring(
                    0, tLocationClass.lastIndexOf( ":" ) );
            }
            else
                tClassName = 
                    "nl.tue.win.riaca.mathematica.codec.core." +
                    tSymbol.getCD() + "Codec";
	    
            try
            {
                if ( tClassLocation.equals( "" ) )
                {
                    Class   tClass   = Class.forName( tClassName );
                    Object  tObject  = tClass.newInstance();
                    CDCodec tCDCodec = (CDCodec) tObject;
          
                    tCDCodec.setParent( this );
                    return tCDCodec.encodeOMObject( tSymbol );
                }
                else
                {
		    URLClassLoader tClassLoader = 
			new URLClassLoader( 
		            new URL[] { new URL( tClassLocation ) } , 
			    this.getClass().getClassLoader() );
		    
                    Class   tClass    = tClassLoader.loadClass( tClassName );
                    Object  tCDObject = tClass.newInstance();
                    CDCodec tCDCodec  = (CDCodec) tCDObject;
		    
                    tCDCodec.setParent( this );
                    return tCDCodec.encodeOMObject( tSymbol );
                }
            }
            catch( MalformedURLException tMalformedURLException )
            {
                throw new CodecEncodeException(
                    "MalformedURLException for codec URL: " + 
                    tClassLocation );
            }
            catch( InstantiationException tInstantiationException )
            {
                throw new CodecEncodeException( 
                    "InstantiationException for codec class: " + tClassName );
            }
            catch( IllegalAccessException tIllegalAccessException )
            {
                throw new CodecEncodeException(
                    "IllegalAccessException for codec class: " + tClassName );
            }
            catch( ClassNotFoundException tClassNotFoundException )
            {
                throw new CodecEncodeException(
                    "ClassNotFoundException for codec class: " + tClassName );
            }
	}
        
        throw new CodecEncodeException( 
            "Unable to encode OMObject: " + fOMObject.toString() );
    }
    
    
    /* -----------------------------------------------------------------------
     *
     *   Decode methods
     *
     * -----------------------------------------------------------------------
     */
    
    
    /**
     * Decodes the Mathematica-result to a XML-encoded OM-object. <p>
     *
     * @since 1.0
     */
    
    public String decode( String fMathematicaResult )
        throws CodecDecodeException
    {
	// System.out.println( "MathematicaCodec.decode" );
        
	StringWriter tStringWriter = new StringWriter();
	OMXMLWriter  tXMLWriter    = new OMXMLWriter( tStringWriter );
        
	OMObject tOMObject = decodeOMObject( fMathematicaResult );
	tXMLWriter.writeObject( tOMObject );
        
	return tStringWriter.toString();
    }
    
    
    /**
     * Decodes the Mathematica-result to an OM-object. <p>
     *
     * To make our life easier, we assume Mathematica is returning its result 
     * in what they call FullForm, it is a prefix tree notation that 
     * facilitates easy-parsing and matching on codecs.
     *
     * @since 1.0
     */
    
    public OMObject decodeOMObject( String fMathematicaResult )
        throws CodecDecodeException
    {
	// System.out.println( "MathematicaCodec.decodeOMObject" );
        
	/*
	 * 1. We first try if it is a composite object,
	 *    eg. OMApplication, OMBinding, OMAttribution, OMError.
	 */
	
	String tOperator = null;
	
	if ( fMathematicaResult.indexOf( '[' ) != -1 )
	{
            tOperator = 
		fMathematicaResult.substring( 
		    0, fMathematicaResult.indexOf( '[' ) );
	}
	if ( tOperator != null  )
	{
	    tOperator = tOperator.trim();
	    String tCDName = (String) mDecodeHashtable.get( tOperator );
             
	    if ( tCDName != null )
	    {
                String tClassName     = "";
                String tClassLocation = "";
		
                if ( mCDs.get( tCDName ) != null )
                {
                    String tLocationClass = (String) mCDs.get( tCDName );
		    
                    tClassName = tLocationClass.substring( 
                        tLocationClass.lastIndexOf( ":" ) + 1 );
		    
                    tClassLocation = tLocationClass.substring(
                        0, tLocationClass.lastIndexOf( ":" ) );
                }
                else
                    tClassName = 
                        "mathematicaCodec." +
                        tCDName + "Codec";
                    
                try
                {
                    if ( tClassLocation.equals( "" ) )
                    {
                        Class   tClass   = Class.forName( tClassName );
                        Object  tObject  = tClass.newInstance();
                        CDCodec tCDCodec = (CDCodec) tObject;
			
                        tCDCodec.setParent( this );
                        return tCDCodec.decodeOMObject( 
                            fMathematicaResult );
                    }
                    else
                    {
			URLClassLoader tClassLoader = 
			    new URLClassLoader( 
				new URL[] { new URL( tClassLocation ) } , 
				this.getClass().getClassLoader() );
			
                        Class   tClass   = 
			    tClassLoader.loadClass( tClassName );
                        Object  tObject  = tClass.newInstance();
                        CDCodec tCDCodec = (CDCodec) tObject;
                            
                        tCDCodec.setParent( this );
                        return tCDCodec.decodeOMObject( 
                            fMathematicaResult );
                    }
                }
                catch( MalformedURLException tMalformedURLException )
                {
                    throw new CodecDecodeException(
                        "MalformedURLException for codec URL: " + 
                        tClassLocation );
                }
                catch( InstantiationException tInstantiationException )
                {
                    throw new CodecDecodeException( 
                        "InstantiationException for codec class: " + 
			tClassName );
                }
                catch( IllegalAccessException tIllegalAccessException )
                {
                    throw new CodecDecodeException(
                        "IllegalAccessException for codec class: " + 
			tClassName );
                }
                catch( ClassNotFoundException tClassNotFoundException )
                {
                    throw new CodecDecodeException(
                        "ClassNotFoundException for codec class: " + 
			tClassName );
                }
	    }
            else
	    {
		/* 
                 * We could not find a CD that handles the operator,
                 * so we generate the symbol to be part of a virtual
                 * Mathematica CD, and create an application for it ;)
                 *
                 * <OMOBJ>
                 *  <OMA>
                 *   <OMS cd="Mathematica" name="Operator"/>
                 *    ...
                 *  </OMA>
                 * </OMOBJ>
                 */
		
		Vector        tVector      = 
		    parseArguments( fMathematicaResult );
		Enumeration   tEnum        = tVector.elements();
		OMApplication tApplication = new OMApplication();
                
		tApplication.addElement( 
		    new OMSymbol( "Mathematica", tOperator ) );
	                
		for( ; tEnum.hasMoreElements(); )
	        {
		    OMObject tOMObject = 
			decodeOMObject( (String) tEnum.nextElement() );
		    
                    tApplication.addElement( tOMObject );
		}
		
                return tApplication;
            }
	}
        
	/*
	 * 2. Trim the string and check if it only contains digits then we
         *    assume it is an OMI.
	 */
	
        String tInteger = fMathematicaResult.trim();
       
	int    tIndex   = 0;
	
	for( tIndex=0; tIndex<tInteger.length(); tIndex++ )
	{
	    if ( Character.isDigit( tInteger.charAt( tIndex ) ) == false &&
                 tInteger.charAt( tIndex ) != '-' )
		break;
	}
	
	if ( tIndex == tInteger.length() )
	{
	    OMInteger tOMInteger = new OMInteger();
	    tOMInteger.setInteger( tInteger );
	    
	    return tOMInteger;
	}
	
	/*
	 * 3. Now we'll try to see if it is a OMF.
	 */
        
        String tFloat = fMathematicaResult.trim();
	
	for( tIndex=0; tIndex<tFloat.length(); tIndex++ )
	{
	    if ( Character.isDigit( tFloat.charAt( tIndex ) ) == false &&
                 tFloat.charAt( tIndex ) != '-'                        &&
		 tFloat.charAt( tIndex ) != '.'                      
		 &&
		 tFloat.charAt( tIndex ) != '`'                        )
		break;
	}
	
	if ( tIndex == tFloat.length() )
	{
	    OMFloat tOMFloat = new OMFloat();
	   
	    tOMFloat.setFloat( tFloat.substring( 0, tFloat.length() - 1 ) );
	    //tOMFloat.setBase( "dec" );
	    
	    return tOMFloat;
	}
	
	/*
	 * 4. Now we'll try to see if it is a string, it should start with
	 *    a quote and end with one.
	 */
	
	String tString = fMathematicaResult.trim();
        
	if ( tString.charAt( 0 ) == '"' &&
	     tString.charAt( tString.length() - 1 ) == '"' )
	{
	    OMString tOMString = new OMString();
	    tOMString.setString( 
	        tString.substring( 1, tString.length() - 1 ) );
	    
	    return tOMString;
	}
        
        /*
         * 5. Now lets try to see if it is a symbol.
         */
        
        String tSymbol = fMathematicaResult.trim();
        
        if ( mDecodeHashtable.get( tSymbol ) != null )
	{
	    String tCDName        = (String) mDecodeHashtable.get( tSymbol );
            String tClassName     = "";
            String tClassLocation = "";
	    
            if ( mCDs.get( tCDName ) != null )
	    {
                String tLocationClass = (String) mCDs.get( tCDName );
		
                tClassName = tLocationClass.substring( 
                    tLocationClass.lastIndexOf( ":" ) + 1 );
                       
                tClassLocation = tLocationClass.substring(
                    0, tLocationClass.lastIndexOf( ":" ) );
            }
            else
                tClassName = 
                    "mathematicaCodec." +
                    tCDName + "Codec";
                    
            try
            {
                if ( tClassLocation.equals( "" ) )
                {
                    Class   tClass   = Class.forName( tClassName );
                    Object  tObject  = tClass.newInstance();
                    CDCodec tCDCodec = (CDCodec) tObject;
		    
                    tCDCodec.setParent( this );
                    return tCDCodec.decodeOMObject( 
                        fMathematicaResult );
                }
                else
                {
		    URLClassLoader tClassLoader = 
			new URLClassLoader( 
			    new URL[] { new URL( tClassLocation ) } , 
			    this.getClass().getClassLoader() );
		    
                    Class   tClass   = tClassLoader.loadClass( tClassName );
                    Object  tObject  = tClass.newInstance();
                    CDCodec tCDCodec = (CDCodec) tObject;
                            
                    tCDCodec.setParent( this );
                    return tCDCodec.decodeOMObject( 
                        fMathematicaResult );
                }
            }
            catch( MalformedURLException tMalformedURLException )
            {
                throw new CodecDecodeException(
                    "MalformedURLException for codec URL: " + 
                    tClassLocation );
            }
            catch( InstantiationException tInstantiationException )
            {
                throw new CodecDecodeException( 
                    "InstantiationException for codec class: " + tClassName );
            }
            catch( IllegalAccessException tIllegalAccessException )
            {
                throw new CodecDecodeException(
                    "IllegalAccessException for codec class: " + tClassName );
            }
            catch( ClassNotFoundException tClassNotFoundException )
            {
                throw new CodecDecodeException(
                   "ClassNotFoundException for codec class: " + tClassName );
            }
        }
        
        /*
	 * 6. Now we assume it is a variable.
	 */
	
	String     tVariable   = fMathematicaResult.trim();
        OMVariable tOMVariable = new OMVariable();
	
	tOMVariable.setName( tVariable );
	
	return tOMVariable;
    }
    
    
    /* -----------------------------------------------------------------------
     *
     *   Utility methods.
     *
     * -----------------------------------------------------------------------
     */
    
    
    /**
     * Creates a list (vector) of arguments for the given FullForm Mathematica 
     * expression. <p>
     *
     * This method is a simple utility method that all the CDCodecs will need, 
     * so it is put here to have a common place and no unnecessary coding 
     * overhead. <p>
     *
     * @since 1.0
     */
    
    public Vector parseArguments( String fMathematicaString )
    {
    	// System.out.println( "MathematicaCodec.parseArguments" );
	
	Vector tArguments = new Vector();
    	String tString = fMathematicaString.substring(
	    fMathematicaString.indexOf( '[' ) + 1,
	    fMathematicaString.lastIndexOf( ']' ) );

	/* 
    	 * Find if the string contains a comma, if it does it is a 
	 * composition. If it doesn't it is a single string.
    	 */
	
	if ( tString.indexOf( ',' ) != -1 )
    	{
	    while( tString.length() > 0 )
    	    {
		int tBracketLevel = 0;
		int i = 0;
		
		for( i=0; i<tString.length() && i>=0; i++ )
		{
		    if ( tString.charAt( i ) == '[' )
			tBracketLevel++;
		    
		    if ( tString.charAt( i ) == ']' )	    
			tBracketLevel--;
		    
		    if ( ( tString.charAt( i ) == ',' ) &&
			 ( tBracketLevel       == 0   )  )
		    {
			String tArgument = tString.substring( 0, i );
			tArguments.addElement( tArgument );
			
			tString = tString.substring( i+1 );
			i = -2;			    
		    }
		}
		
		if ( i != -1 )
		{
		    tArguments.addElement( tString );
		    tString = "";
                }
    	    }
    	}
    	else
	    tArguments.addElement( tString );
	
    	return tArguments;
    }
    
    
    /**
     * Adds a CD to the Codec. <p>
     *
     * @param fName the name of the CD.
     * @param fLocation the URL of the CD, if none is given it is assume to be 
     *                  on the classpath
     * @param fClassname the Codec of the CD.
     * @since 1.0
     */
    
    public void addCD( String fName, String fLocation, String fClassName )
        throws CodecException
    {
        super.addCD( fName, fLocation, fClassName );
        
        // System.out.println( "MathematicaCodec.addCD" );
	
        try
        {
            if ( fLocation.equals( "" ) )
            {
                Class   tClass   = Class.forName( fClassName );
                Object  tObject  = tClass.newInstance();
                CDCodec tCDCodec = (CDCodec) tObject;
                        
                tCDCodec.setParent( this );
                        
                if ( tCDCodec instanceof MathematicaCDCodec )
                {
                    MathematicaCDCodec tCodec = (MathematicaCDCodec) tCDCodec;
                    tCodec.registerDecoders();
                }
            }
            else
            {
		URLClassLoader tClassLoader = 
		    new URLClassLoader( 
		        new URL[] { new URL( fLocation ) } , 
			this.getClass().getClassLoader() );
		
                Class tClass = tClassLoader.loadClass( fClassName );
                Object  tObject = tClass.newInstance();
                CDCodec tCDCodec = (CDCodec) tObject;
                            
                tCDCodec.setParent( this );
            
                if ( tCDCodec instanceof MathematicaCDCodec )
                {
                    MathematicaCDCodec tCodec = (MathematicaCDCodec) tCDCodec;
                    tCodec.registerDecoders();
                }
            }
        }
        catch( Exception tException )
        {
            throw new CodecException( tException.getMessage() );
        }
    }


    /**
     * Removes a CD from the Codec. <p>
     *
     * @param fName the name of the CD.
     * @since 1.0
     */

    public void removeCD( String fName )
        throws CodecException
    {
        System.out.println( "MathematicaCodec.removeCD" );

        try
        {
            String tClassName     = "";
            String tClassLocation = "";
                     
            if ( mCDs.get( fName ) != null )
            {
                String tLocationClass = (String) mCDs.get( fName );
                        
                tClassName = tLocationClass.substring( 
                    tLocationClass.lastIndexOf( ":" ) + 1 );
                       
                tClassLocation = tLocationClass.substring(
                    0, tLocationClass.lastIndexOf( ":" ) );
            }
            else
                tClassName = 
                    "mathematicaCodec." +
                    fName + "Codec";

            if ( tClassLocation.equals( "" ) )
            {
                Class   tClass   = Class.forName( tClassName );
                Object  tObject  = tClass.newInstance();
                CDCodec tCDCodec = (CDCodec) tObject;
            
                tCDCodec.setParent( this );
            
                if ( tCDCodec instanceof MathematicaCDCodec )
                {
                    MathematicaCDCodec tCodec = (MathematicaCDCodec) tCDCodec;
                    tCodec.deregisterDecoders();
                }
            }
            else
            {
		URLClassLoader tClassLoader = 
		    new URLClassLoader( 
		        new URL[] { new URL( tClassLocation ) } , 
			this.getClass().getClassLoader() );
		
                Class   tClass   = tClassLoader.loadClass( tClassName );
                Object  tObject  = tClass.newInstance();
                CDCodec tCDCodec = (CDCodec) tObject;
                            
                tCDCodec.setParent( this );
            
                if ( tCDCodec instanceof MathematicaCDCodec )
                {
                    MathematicaCDCodec tCodec = (MathematicaCDCodec) tCDCodec;
                    tCodec.deregisterDecoders();
                }
            }
        }
        catch( Exception tException )
        {
            throw new CodecException( tException.getMessage() );
        }

        super.removeCD( fName );
    }


    /**
     * @since 1.0
     */

    public void addDecoder( String fName, String fCD )
    {
        System.out.println( "MathematicaCodec.addDecoder" );

        mDecodeHashtable.put( fName, fCD );
    }


    /**
     * @since 1.0
     */

    public void removeDecoder( String fName )
    {
        System.out.println( "MathematicaCodec.removeDecoder" );

        mDecodeHashtable.remove( fName );
    }
}

