/*
 * $Id: arith1Codec.java,v 1.10 2002/11/25 10:22:50 mriem Exp $
 *
 * Copyright (c) 1999-2002, RIACA, Technische Universiteit Eindhoven (TU/e).
 * All Rights Reserved.
 *
 * The contents of this file are subject to the RIACA Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.riaca.win.tue.nl/
 */

package mathematicaCodec;

import nl.tue.win.riaca.openmath.codec.*;
import nl.tue.win.riaca.openmath.lang.*;
import java.util.*;


/**
 * A CD-codec that handles the OpenMath 'arith1' CD. <p>
 *
 * @author Manfred Riem (mriem@win.tue.nl)
 * @version 1.3
 */

public class arith1Codec
    extends CDCodec
{
    /**
     * Stores the names in this codec. <p>
     *
     * @since 1.0
     */
   
    public String[] sNames = 
    {
        "abs",
        "divide",
        "gcd",
        "lcm",
        "minus",
        "plus",
        "power",
        "product",
        "root",
        "sum",
        "times",
        "unary_minus"
    };


    /**
     * Stores the Mathematica equivalents to the OpenMath symbols. <p>
     *
     * @since 1.0
     */

    public String[] sMathNames =
    {
        "Abs",
        "Divide",
        "GCD",
        "LCM",
        "Subtract",
        "Plus",
        "Power",
        "Product",
        "Sqrt",
        "Sum",
        "Times",
        "Minus"
    };


    /**
     * Constructor. <p>
     *
     * @since 1.0
     */

    public arith1Codec()
    {
        super();
        
        // System.out.println( "arith1Codec.<init>" );
    }

 
    /**
     * Constructor. <p>
     *
     * @since 1.0
     */
   
    public arith1Codec( Codec fParent )
    {
        super();

	mParent = fParent;
	
	// System.out.println( "arith1Codec.<init>" );
    }
  
  
    /**
     * Encodes the given OMObject to Mathematica-syntax. <p>
     *
     * @since 1.0
     */
   
    public String encodeOMObject( OMObject fOMObject )
        throws  CodecEncodeException
    {
        // System.out.println( "arith1Codec.encode" );
	    
	OMApplication tApplication = (OMApplication) fOMObject;
	OMObject      tOMObject    = tApplication.firstElement();

	if ( tOMObject instanceof OMSymbol )
	{
	    OMSymbol tOMSymbol = (OMSymbol) tOMObject;
	        
	    switch( getIndex( tOMSymbol.getName() ) )
	    {
	        case 0  : return encodeAbs( fOMObject );
		case 1  : return encodeDivide( fOMObject );
		case 2  : return encodeGcd( fOMObject );
		case 3  : return encodeLcm( fOMObject );
		case 4  : return encodeMinus( fOMObject );
		case 5  : return encodePlus( fOMObject );
		case 6  : return encodePower( fOMObject );
		case 7  : return encodeProduct( fOMObject );
		case 8  : return encodeRoot( fOMObject );
		case 9  : return encodeSum( fOMObject );
		case 10 : return encodeTimes( fOMObject );
		case 11 : return encodeUnaryMinus( fOMObject );
	    }
	}
        
	throw new CodecEncodeException( "Unable to encode: " + fOMObject );
    }


    /**
     * Decodes the given Mathematica-syntax to a OMObject. <p>
     *
     * @since 1.0
     */
  
    public OMObject decodeOMObject( String fMathematicaSyntax )
        throws CodecDecodeException
    {
	// System.out.println( "arith1Codec.decode" );
	
	String tOperator = 
	    fMathematicaSyntax.substring( 
                0, fMathematicaSyntax.indexOf( '[' ) );
	
	String tArguments =
	    fMathematicaSyntax.substring( fMathematicaSyntax.indexOf( '[' ) );
	
	if ( tOperator != null )
	{
	    tOperator = tOperator.trim();
	    
	    switch( getMathIndex( tOperator ) )
	    {
	        case 0  : return decodeAbs( tArguments );
		case 1  : return decodeDivide( tArguments );
      	        case 2  : return decodeGcd( tArguments );
	        case 3  : return decodeLcm( tArguments );
	        case 4  : return decodeMinus( tArguments );
	        case 5  : return decodePlus( tArguments );
	        case 6  : return decodePower( tArguments );
	        case 7  : return decodeProduct( tArguments );
	        case 8  : return decodeRoot( tArguments );
	        case 9  : return decodeSum( tArguments );
	        case 10 : return decodeTimes( tArguments );
	        case 11 : return decodeUnaryMinus( tArguments );
	    }
	}
	
	throw new CodecDecodeException( 
            "Unable to decode: " + fMathematicaSyntax );
    }
    
    
    /**
     * Gets the index for the name. <p>
     *
     * @since 1.0
     */
    
    private int getIndex( String fName )
    {
	// System.out.println( "arith1Codec.getIndex" );
	
	for( int i=0; i<sNames.length; i++ )
	{
	    if ( fName.equals( sNames[i] ) )
		return i;
	}
	
	return -1;
    }


    /**
     * Gets the index for the Mathematica operator. <p>
     *
     * @since 1.0
     */
    
    private int getMathIndex( String fOperator )
    {
	// System.out.println( "arith1Codec.getMathIndex" );
	
	for( int i=0; i<sNames.length; i++ )
	{
	    if ( fOperator.equals( sMathNames[i] ) )
		return i;
	}
	
	return -1;
    }
    
    
    /**
     * Encodes the arith1.abs. <p>
     *
     * @since 1.0
     */
    
    public String encodeAbs( OMObject fOMObject )
        throws  CodecEncodeException
    {
	// System.out.println( "arith1Codec.encodeAbs" );
	
	StringBuffer  tResult      = new StringBuffer();
	OMApplication tApplication = (OMApplication) fOMObject;
	
	tResult.append( "Abs[" );
	
	if ( tApplication.getElementAt( 1 ) != null )
	{
    	    tResult.append( 
		mParent.encodeOMObject( tApplication.getElementAt( 1 ) ) );
	} 
	else
	    throw new CodecEncodeException( "Unable to encode: " + fOMObject );
	
        tResult.append( "]" );
	return tResult.toString();
    }
    
    
    /**
     * Encodes the arith1.divide. 
     *
     * @since 1.0
     */
    
    public String encodeDivide( OMObject fOMObject )
        throws  CodecEncodeException
    {
	// System.out.println( "arith1Codec.encodeDivide" );
	
	StringBuffer  tResult      = new StringBuffer();
	OMApplication tApplication = (OMApplication) fOMObject;
	
	tResult.append( "Divide[" );
	
	if ( tApplication.getElementAt( 1 ) != null )
	{
	    tResult.append(
		 mParent.encodeOMObject( tApplication.getElementAt( 1 ) ) );
	}
	else
	    throw new CodecEncodeException( "Unable to encode: " + fOMObject );
	
	tResult.append( "," );
	
        if ( tApplication.getElementAt( 2 ) != null )
        {
	    tResult.append(
		 mParent.encodeOMObject( tApplication.getElementAt( 2 ) ) );
	}
	else
	    throw new CodecEncodeException( "Unable to encode: " + fOMObject );
	
	tResult.append( "]" );
	return tResult.toString();
    }
    
    
    /**
     * Encodes the arith1.gcd 
     *
     * @since 1.0
     */
    
    public String encodeGcd( OMObject fOMObject )
        throws  CodecEncodeException
    {
	// System.out.println( "arith1Codec.encodeGcd" );
     
	StringBuffer  tResult      = new StringBuffer();
	OMApplication tApplication = (OMApplication) fOMObject;
	Enumeration   tEnum        = tApplication.getElements().elements();
	
	tEnum.nextElement();
	
	if ( tApplication.getLength() >= 3 )
	{
	    tResult.append( "GCD[" );
	    
	    for( ; tEnum.hasMoreElements(); )
	    {
		tResult.append( 
		    mParent.encodeOMObject( (OMObject) tEnum.nextElement() ) );
		
		if ( tEnum.hasMoreElements() )
		    tResult.append( "," );
	    }
	    
	    tResult.append( "]" );
	    
	    return tResult.toString();
	}
	
        throw new CodecEncodeException( "Unable to encode: " + fOMObject );
    }


    /**
     * Encodes the arith1.lcm
     *
     * @since 1.0
     */
   
    public String encodeLcm( OMObject fOMObject )
        throws  CodecEncodeException
    {
	// System.out.println( "arith1Codec.encodeLcm" );
        
	StringBuffer  tResult      = new StringBuffer();
	OMApplication tApplication = (OMApplication) fOMObject;
	Enumeration   tEnum        = tApplication.getElements().elements();
	
	tEnum.nextElement();
	
	if ( tApplication.getLength() >= 3 )
	{
	    tResult.append( "LCM[" );
	    
	    for( ; tEnum.hasMoreElements(); )
	    {
		tResult.append( 
		    mParent.encodeOMObject( (OMObject) tEnum.nextElement() ) );
		
		if ( tEnum.hasMoreElements() )
		    tResult.append( "," );
	    }
	    
	    tResult.append( "]" );
	    
	    return tResult.toString();
	}
	
        throw new CodecEncodeException( "Unable to encode: " + fOMObject );
    }
    
    
    /**
     * Encodes the arith1.minus. 
     *
     * @since 1.0
     */
    
    public String encodeMinus( OMObject fOMObject )
        throws  CodecEncodeException
    {
	// System.out.println( "arith1Codec.encodeMinus" );
	
	StringBuffer  tResult      = new StringBuffer();
	OMApplication tApplication = (OMApplication) fOMObject;
	tResult.append( "Subtract[" );
	
	if ( tApplication.getElementAt( 1 ) != null )
	{
	    tResult.append(
		 mParent.encodeOMObject( tApplication.getElementAt( 1 ) ) );
	}
	else
	    throw new CodecEncodeException( "Unable to encode: " + fOMObject );
	
	tResult.append( "," );
	
        if ( tApplication.getElementAt( 2 ) != null )
        {
	    tResult.append(
		 mParent.encodeOMObject( tApplication.getElementAt( 2 ) ) );
	}
	else
	    throw new CodecEncodeException( "Unable to encode: " + fOMObject );
	
	tResult.append( "]" );
	return tResult.toString();
    }
    
    
    /**
     * Encodes the arith1-plus. <p> 
     *
     * @since 1.0
     */
    
    public String encodePlus( OMObject fOMObject )
        throws CodecEncodeException
    {
	// System.out.println( "arith1Codec.encodePlus" );
	
	StringBuffer  tResult      = new StringBuffer();
	OMApplication tApplication = (OMApplication) fOMObject;
	Enumeration   tEnum        = tApplication.getElements().elements();
	
	tEnum.nextElement();
	
	if ( tApplication.getLength() >= 3 )
	{
	    tResult.append( "Plus[" );
	    
	    for( ; tEnum.hasMoreElements(); )
	    {
		tResult.append( 
		    mParent.encodeOMObject( (OMObject) tEnum.nextElement() ) );
		
		if ( tEnum.hasMoreElements() )
		    tResult.append( "," );
	    }
	    
	    tResult.append( "]" );
	    
	    return tResult.toString();
	}
	
        throw new CodecEncodeException( "Unable to encode: " + fOMObject );
    }
    
    
    /**
     * Encodes the arith1.power. 
     *
     * @since 1.0
     */
    
    public String encodePower( OMObject fOMObject )
        throws  CodecEncodeException
    {
	// System.out.println( "arith1Codec.encodePower" );
	
	StringBuffer  tResult      = new StringBuffer();
	OMApplication tApplication = (OMApplication) fOMObject;
	
	boolean tIsMatrix = false; 
        
        if ( tApplication.getElementAt( 1 ) != null )
        {
            tIsMatrix = IsMatrixOrVector( tApplication.getElementAt( 1 ) );
        }
        
        if ( tIsMatrix )
        {
            tResult.append( "MatrixPower[" );
        }
        else
        {
            tResult.append( "Power[" );
        }
	
	if ( tApplication.getElementAt( 1 ) != null )
	{
	    tResult.append(
		 mParent.encodeOMObject( tApplication.getElementAt( 1 ) ) );
	}
	else
	    throw new CodecEncodeException( "Unable to encode: " + fOMObject );

	tResult.append( "," );

        if ( tApplication.getElementAt( 2 ) != null )
        {
	    tResult.append(
		 mParent.encodeOMObject( tApplication.getElementAt( 2 ) ) );
	}
	else
	    throw new CodecEncodeException( "Unable to encode: " + fOMObject );

	tResult.append( "]" );
	return tResult.toString();
    }


    /**
     * Encodes the arith1.product.
     *
     * @since 1.0
     */
   
    public String encodeProduct( OMObject fOMObject )
        throws  CodecEncodeException
    {
	// System.out.println( "arith1Codec.encodeProduct" );
     
	StringBuffer  tResult      = new StringBuffer();
	OMApplication tApplication = (OMApplication) fOMObject;
	
	tResult.append( "Product[" );
	
	if ( tApplication.getElementAt( 2 ) != null )
	{
	    OMBinding tBinding = (OMBinding) tApplication.getElementAt( 2 );
	    
	    tResult.append(
		 mParent.encodeOMObject( tBinding.getBody() ) );
	}
	else
	    throw new CodecEncodeException( "Unable to encode: " + fOMObject );
	
	tResult.append( ",List[" );
	
	if ( tApplication.getElementAt( 2 ) != null )
	{
	    OMBinding   tBinding = (OMBinding) tApplication.getElementAt( 2 );
	    Enumeration tEnum    = tBinding.getVariables().elements();
	    
	    for( ; tEnum.hasMoreElements() ; )
	    {
		tResult.append( 
		    mParent.encodeOMObject(
			(OMObject) tEnum.nextElement() ) );
		
		tResult.append( "," );
		
		if ( tEnum.hasMoreElements() )
		    throw new CodecEncodeException( 
                        "Unable to encode: " + fOMObject );
	    }
	}
	
        if ( tApplication.getElementAt( 1 ) != null )
        {
	    OMApplication tIntervalApplication = 
		(OMApplication) tApplication.getElementAt( 1 );
	    
	    OMSymbol tIntervalSymbol = 
		(OMSymbol) tIntervalApplication.getElementAt( 0 );
	    
	    if ( tIntervalSymbol.getCD().equals( "interval1" ) &&
		 tIntervalSymbol.getName().equals( "integer_interval" ) )
	    {
		tResult.append(
		    mParent.encodeOMObject( 
		        tIntervalApplication.getElementAt( 1 ) ) );
		
		tResult.append( "," );
		
		tResult.append(
		    mParent.encodeOMObject( 
                        tIntervalApplication.getElementAt( 2 ) ) );
	    }
	    else
		throw new CodecEncodeException( 
                    "Unable to encode: " + fOMObject );
	}
	else
	    throw new CodecEncodeException( 
                "Unable to encode: " + fOMObject );
	
	tResult.append( "]]" );
	return tResult.toString();
    }
    
    
    /**
     * Encodes the arith1.root.
     *
     * @since 1.0
     */
   
    public String encodeRoot( OMObject fOMObject )
        throws  CodecEncodeException
    {
	// System.out.println( "arith1Codec.encodeRoot" );
     
	StringBuffer  tResult      = new StringBuffer();
	OMApplication tApplication = (OMApplication) fOMObject;
    
	tResult.append( "Power[" );
	
	if ( tApplication.getElementAt( 1 ) != null )
	{
	    tResult.append(
		 mParent.encodeOMObject( tApplication.getElementAt( 1 ) ) );
	}
	else
	    throw new CodecEncodeException( "Unable to encode: " + fOMObject );
	
	tResult.append( "," );
	
        if ( tApplication.getElementAt( 2 ) != null )
        {
	    tResult.append( "Divide[1," );
	    tResult.append(
		 mParent.encodeOMObject( tApplication.getElementAt( 2 ) ) );
	    tResult.append( "]" );
	}
	else
	    throw new CodecEncodeException( "Unable to encode: " + fOMObject );
	
	tResult.append( "]" );
	return tResult.toString();
    }
    
    
    /**
     * Encodes the arith1.sum. 
     *
     * @since 1.0
     */
    
    public String encodeSum( OMObject fOMObject )
        throws  CodecEncodeException
    {
	// System.out.println( "arith1Codec.encodeSum" );
	
	StringBuffer  tResult      = new StringBuffer();
	OMApplication tApplication = (OMApplication) fOMObject;
    
	tResult.append( "Sum[" );
	
	if ( tApplication.getElementAt( 2 ) != null )
	{
	    OMBinding tBinding = (OMBinding) tApplication.getElementAt( 2 );

	    tResult.append(
		 mParent.encodeOMObject( tBinding.getBody() ) );
	}
	else
	    throw new CodecEncodeException( "Unable to encode: " + fOMObject );

	tResult.append( ",List[" );
	
	if ( tApplication.getElementAt( 2 ) != null )
	{
	    OMBinding   tBinding = (OMBinding) tApplication.getElementAt( 2 );
	    Enumeration tEnum    = tBinding.getVariables().elements();
	    
	    for( ; tEnum.hasMoreElements() ; )
	    {
		tResult.append( 
		    mParent.encodeOMObject(
			(OMObject) tEnum.nextElement() ) );
		
		tResult.append( "," );
		
		if ( tEnum.hasMoreElements() )
		    throw new CodecEncodeException( "Unable to encode: " + 
						    fOMObject );
	    }
	}
	
        if ( tApplication.getElementAt( 1 ) != null )
        {
	    OMApplication tIntervalApplication = 
		(OMApplication) tApplication.getElementAt( 1 );

	    OMSymbol tIntervalSymbol = 
		(OMSymbol) tIntervalApplication.getElementAt( 0 );
 
	    if ( tIntervalSymbol.getCD().equals( "interval1" ) &&
		 tIntervalSymbol.getName().equals( "integer_interval" ) )
	    {
		tResult.append(
		    mParent.encodeOMObject( 
                        tIntervalApplication.getElementAt( 1 ) ) );
	
		tResult.append( "," );

		tResult.append(
		    mParent.encodeOMObject( 
                        tIntervalApplication.getElementAt( 2 ) ) );
	    }
	    else
		throw new CodecEncodeException( "Unable to encode: " + 
						fOMObject );
	}
	else
	    throw new CodecEncodeException( "Unable to encode: " + fOMObject );

	tResult.append( "]]" );
	return tResult.toString();
    }


    /**
     * Encodes the arith1.times.
     *
     * @since 1.0
     */
       
    public String encodeTimes( OMObject fOMObject )
        throws  CodecEncodeException
    {
	// System.out.println( "arith1Codec.encodeTimes" );
     	
        StringBuffer  tResult      = new StringBuffer();
	OMApplication tApplication = (OMApplication) fOMObject;
	Enumeration   tEnum        = tApplication.getElements().elements();
	
        /**
         * We have to distinguish the case when the arguments are matrices 
	 * or vectors. In that case instead of Times[] we need to use Dot[].
         */
        
        boolean tIsMatrix = false;
        
        if ( tApplication.getElementAt( 1 ) != null  )
            tIsMatrix = IsMatrixOrVector( tApplication.getElementAt( 1 ) );
        
        if ( tIsMatrix )
            tResult.append( "Dot[" );
        else
            tResult.append( "Times[" );
		
	if ( tApplication.getLength() >= 3 )
	{
	    tEnum.nextElement();
	    
	    for( ; tEnum.hasMoreElements(); )
	    {
		tResult.append( 
		    mParent.encodeOMObject( (OMObject) tEnum.nextElement() ) );
		
		if ( tEnum.hasMoreElements() )
		    tResult.append( "," );
	    }
	    
	    tResult.append( "]" );
	    
	    return tResult.toString();
	}
	
	throw new CodecEncodeException( "Unable to encode: " + fOMObject );
    }


    /**
     * Encodes the arith1.unary_minus.
     *
     * @since 1.0
     */
   
    public String encodeUnaryMinus( OMObject fOMObject )
        throws  CodecEncodeException
    {
	// System.out.println( "arith1Codec.encodeUnaryMinus" );
	
	StringBuffer  tResult      = new StringBuffer();
	OMApplication tApplication = (OMApplication) fOMObject;
	
	tResult.append( "Minus[" );
	
	if ( tApplication.getElementAt( 1 ) != null )
	{
    	    tResult.append( 
		mParent.encodeOMObject( tApplication.getElementAt( 1 ) ) );
	} 
	else
	    throw new CodecEncodeException( "Unable to encode: " + fOMObject );
	
        tResult.append( "]" );
	return tResult.toString();
    }
    
    
    /**
     * Decodes the Abs Mathematica expression. <p>
     *
     * @since 1.0
     */
    
    public OMObject decodeAbs( String fMathematicaSyntax )
        throws  CodecDecodeException
    {
	// System.out.println( "arith1Codec.decodeAbs" );
	
	OMApplication tOMApplication = new OMApplication();
	
	tOMApplication.addElement( new OMSymbol( "arith1", "abs" ) );
	
	MathematicaCodec tParentCodec = (MathematicaCodec) mParent;
	Vector tVector = tParentCodec.parseArguments( fMathematicaSyntax );
	
	if ( tVector.size() == 1 )
	{
	    OMObject tOMObject = 
		tParentCodec.decodeOMObject( 
                    (String) tVector.elementAt( 0 ) );
	    
	    tOMApplication.addElement( tOMObject );
	}
	else
	    throw new CodecDecodeException( "Unable to decode: " + 
					    fMathematicaSyntax );
	
	return tOMApplication;
    }
    
    
    /**
     * Decodes the Mathematica (Divide) expression. <p>
     *
     * @since 1.0
     */
    
    public OMObject decodeDivide( String fMathematicaSyntax )
        throws  CodecDecodeException
    {
	// System.out.println( "arith1Codec.decodeDivide" );
	
	OMApplication tOMApplication = new OMApplication();
	
	tOMApplication.addElement( new OMSymbol( "arith1", "divide" ) );
	
	MathematicaCodec tParentCodec = (MathematicaCodec) mParent;
	Vector tVector = tParentCodec.parseArguments( fMathematicaSyntax );
	
	if ( tVector.size() == 2 )
	{
	    OMObject tOMObject = 
		tParentCodec.decodeOMObject( 
                    (String) tVector.elementAt( 0 ) );
	    
	    tOMApplication.addElement( tOMObject );
	    
	    tOMObject = tParentCodec.decodeOMObject(
		    (String) tVector.elementAt( 1 ) );		
	    
	    tOMApplication.addElement( tOMObject );
	}
	else
	    throw new CodecDecodeException( "Unable to decode: " + 
					    fMathematicaSyntax );
	
	return tOMApplication;
    }


    /**
     * Decodes the Mathematica (GCD) expression. <p>
     *
     * @since 1.0
     */

    public OMObject decodeGcd( String fMathematicaSyntax )
        throws  CodecDecodeException
    {
	// System.out.println( "arith1Codec.decodeGcd" );
	
	OMApplication tOMApplication = new OMApplication();
	
	tOMApplication.addElement( new OMSymbol( "arith1", "gcd" ) );
	
	MathematicaCodec tParentCodec = (MathematicaCodec) mParent;
	Vector tVector = tParentCodec.parseArguments( fMathematicaSyntax );
	Enumeration tEnum = tVector.elements();
	
	for( ; tEnum.hasMoreElements(); )
	{
	    OMObject tOMObject = 
		tParentCodec.decodeOMObject( 
                    (String) tEnum.nextElement() );
	    
	    if ( tOMObject != null )
		tOMApplication.addElement( tOMObject );
	    else
		throw new CodecDecodeException( "Unable to decode: " + 
						fMathematicaSyntax );
	}
	
	return tOMApplication;
    }
    
    
    /**
     * Decodes the Mathematica (LCM) expression. <p>
     *
     * @since 1.0
     */
    
    public OMObject decodeLcm( String fMathematicaSyntax )
        throws CodecDecodeException
    {
        // System.out.println( "arith1Codec.decodeLcm" );
        
	OMApplication tOMApplication = new OMApplication();
        
	tOMApplication.addElement( new OMSymbol( "arith1", "lcm" ) );
	
	MathematicaCodec tParentCodec = (MathematicaCodec) mParent;
	Vector           tVector      = 
	    tParentCodec.parseArguments( fMathematicaSyntax );
	
	Enumeration      tEnum        = tVector.elements();
	    
	for( ; tEnum.hasMoreElements(); )
	{
	    OMObject tOMObject = 
		tParentCodec.decodeOMObject( 
                    (String) tEnum.nextElement() );
            
	    if ( tOMObject != null )
		tOMApplication.addElement( tOMObject );
	    else
		throw new CodecDecodeException( "Unable to decode: " + 
						fMathematicaSyntax );
	}
	
	return tOMApplication;
    }
    
    
    /**
     * Decodes the Mathematica (Subtract) expression. <p>
     *
     * @since 1.0
     */
    
    public OMObject decodeMinus( String fMathematicaSyntax )
            throws  CodecDecodeException
        {
         //System.out.println( "Ons and Cvetan" );
       
        MathematicaCodec tParentCodec = (MathematicaCodec) mParent;
        Vector tVector = tParentCodec.parseArguments( fMathematicaSyntax );       
        int cnt = tVector.size()-1; 
        OMApplication tOMApplicationParent = new OMApplication();
        tOMApplicationParent.addElement( new OMSymbol( "arith1", "minus" ) );
        OMObject tOMObject = tParentCodec.decodeOMObject(
                (String) tVector.elementAt( cnt ) );
        tOMApplicationParent.addElement( tOMObject );
        OMApplication tOMApplicationTmp = tOMApplicationParent;
        cnt--;
  	OMApplication tOMApplication = new OMApplication();
        
    	tOMApplication.addElement( new OMSymbol( "arith1", "minus" ) );
        if ( tVector.size() == 2 )
    	{
    	     tOMObject = 
    		tParentCodec.decodeOMObject( 
                        (String) tVector.elementAt( 0 ) );
     
    	    tOMApplication.addElement( tOMObject );

    	    tOMObject = tParentCodec.decodeOMObject(
    		    (String) tVector.elementAt( 1 ) );		
    	    
    	    tOMApplication.addElement( tOMObject );
    	    return tOMApplication ;
    	}
        while (cnt > 0) {           
            OMApplication tOMApplicationChild = new OMApplication();;
            tOMApplicationChild.addElement( new OMSymbol( "arith1", "minus" ) );
            tOMObject = tParentCodec.decodeOMObject(
                                (String) tVector.elementAt( cnt ) );
            tOMApplicationChild.addElement( tOMObject );
            cnt--;                 
            if(cnt == 0) {
                tOMObject = tParentCodec.decodeOMObject(
                        (String) tVector.elementAt( cnt ) );
                tOMApplicationChild.addElement( tOMObject );
                tOMApplicationTmp.addElement( tOMApplicationChild );
                return tOMApplicationParent;
            }
            tOMApplicationTmp.addElement( tOMApplicationChild );       
            tOMApplicationTmp = tOMApplicationChild;
        }                   
        return tOMApplicationParent;
}


    /**
     * Decodes the Mathematica (Plus) expression. <p>
     *
     * @since 1.0
     */

    public OMObject decodePlus( String fMathematicaSyntax )
            throws  CodecDecodeException
        {
         //System.out.println( "Ons and Cvetan" );
       
        MathematicaCodec tParentCodec = (MathematicaCodec) mParent;
        Vector tVector = tParentCodec.parseArguments( fMathematicaSyntax );       
        int cnt = tVector.size()-1; 
        OMApplication tOMApplicationParent = new OMApplication();
        tOMApplicationParent.addElement( new OMSymbol( "arith1", "plus" ) );
        OMObject tOMObject = tParentCodec.decodeOMObject(
                (String) tVector.elementAt( cnt ) );
        tOMApplicationParent.addElement( tOMObject );
        OMApplication tOMApplicationTmp = tOMApplicationParent;
        cnt--;
  	OMApplication tOMApplication = new OMApplication();
        
    	tOMApplication.addElement( new OMSymbol( "arith1", "plus" ) );
        if ( tVector.size() == 2 )
    	{
    	     tOMObject = 
    		tParentCodec.decodeOMObject( 
                        (String) tVector.elementAt( 0 ) );
     
    	    tOMApplication.addElement( tOMObject );

    	    tOMObject = tParentCodec.decodeOMObject(
    		    (String) tVector.elementAt( 1 ) );		
    	    
    	    tOMApplication.addElement( tOMObject );
    	    return tOMApplication ;
    	}
        while (cnt > 0) {           
            OMApplication tOMApplicationChild = new OMApplication();;
            tOMApplicationChild.addElement( new OMSymbol( "arith1", "plus" ) );
            tOMObject = tParentCodec.decodeOMObject(
                                (String) tVector.elementAt( cnt ) );
            tOMApplicationChild.addElement( tOMObject );
            cnt--;                 
            if(cnt == 0) {
                tOMObject = tParentCodec.decodeOMObject(
                        (String) tVector.elementAt( cnt ) );
                tOMApplicationChild.addElement( tOMObject );
                tOMApplicationTmp.addElement( tOMApplicationChild );
                return tOMApplicationParent;
            }
            tOMApplicationTmp.addElement( tOMApplicationChild );       
            tOMApplicationTmp = tOMApplicationChild;
        }                   
        return tOMApplicationParent;
}


    /**
     * Decodes the Mathematica (Power) expression. <p>
     *
     * @since 1.0
     */

    public OMObject decodePower( String fMathematicaSyntax )
        throws  CodecDecodeException
    {
	// System.out.println( "arith1Codec.decodePower" );

	OMApplication tOMApplication = new OMApplication();
       
	tOMApplication.addElement( new OMSymbol( "arith1", "power" ) );
	
	MathematicaCodec tParentCodec = (MathematicaCodec) mParent;
	Vector tVector = tParentCodec.parseArguments( fMathematicaSyntax );

	if ( tVector.size() == 2 )
	{
	    OMObject tOMObject = 
		tParentCodec.decodeOMObject( 
                    (String) tVector.elementAt( 0 ) );
 
	    tOMApplication.addElement( tOMObject );

	    tOMObject = tParentCodec.decodeOMObject(
		    (String) tVector.elementAt( 1 ) );		
	    
	    tOMApplication.addElement( tOMObject );
	}
	else
	    throw new CodecDecodeException( "Unable to decode: " + 
					    fMathematicaSyntax );
	
	return tOMApplication;
    }


    /**
     * Decodes the Mathematica (Product) expression. <p>
     *
     * @since 1.0
     */

    public OMObject decodeProduct( String fMathematicaSyntax )
        throws  CodecDecodeException
    {
	// System.out.println( "arith1Codec.decodeProduct" );


	
	MathematicaCodec tParentCodec = (MathematicaCodec) mParent;
	Vector tVector = tParentCodec.parseArguments( fMathematicaSyntax );
	OMApplication tOMApplication = new OMApplication();
    
	tOMApplication.addElement( new OMSymbol( "arith1", "product" ) );

	if ( tVector.size() == 2 )
	{
	    OMObject tOMObject = 
		tParentCodec.decodeOMObject( 
                    (String) tVector.elementAt( 0 ) );
 
	    tOMApplication.addElement( tOMObject );

	    tOMObject = tParentCodec.decodeOMObject(
		    (String) tVector.elementAt( 1 ) );		
	    
	    tOMApplication.addElement( tOMObject );
	}
	else
	    throw new CodecDecodeException( "Unable to decode: " + 
					    fMathematicaSyntax );
   
	return tOMApplication;
    }


    /**
     * Decodes the Mathematica (Sqrt) expression. <p>
     *
     * @since 1.0
     */

    public OMObject decodeRoot( String fMathematicaSyntax )
        throws  CodecDecodeException
    {
	// System.out.println( "arith1Codec.decodeRoot" );

	OMApplication tOMApplication = new OMApplication();
       
	tOMApplication.addElement( new OMSymbol( "arith1", "root" ) );
	
	MathematicaCodec tParentCodec = (MathematicaCodec) mParent;
	Vector tVector = tParentCodec.parseArguments( fMathematicaSyntax );

	if ( tVector.size() == 1 )
	{
	    OMObject tOMObject = 
		tParentCodec.decodeOMObject( 
                    (String) tVector.elementAt( 0 ) );
 
	    tOMApplication.addElement( tOMObject );

	    tOMObject = new OMInteger( 2 ); 
	    tOMApplication.addElement( tOMObject );
	}
	else
	    throw new CodecDecodeException( "Unable to decode: " + 
					    fMathematicaSyntax );
   
	return tOMApplication;
    }


    /**
     * Decodes the Mathematica (Sum) expression. <p>
     *
     * @since 1.0
     */

    public OMObject decodeSum( String fMathematicaSyntax )
        throws  CodecDecodeException
    {
	// System.out.println( "arith1Codec.decodeSum" );

	OMApplication tOMApplication = new OMApplication();
       
	tOMApplication.addElement( new OMSymbol( "arith1", "sum" ) );
	
	MathematicaCodec tParentCodec = (MathematicaCodec) mParent;
	Vector tVector = tParentCodec.parseArguments( fMathematicaSyntax );

	if ( tVector.size() == 2 )
	{
	    OMObject tOMObject = 
		tParentCodec.decodeOMObject( 
                    (String) tVector.elementAt( 0 ) );
 
	    tOMApplication.addElement( tOMObject );

	    tOMObject = tParentCodec.decodeOMObject(
		    (String) tVector.elementAt( 1 ) );		
	    
	    tOMApplication.addElement( tOMObject );
	}
	else
	    throw new CodecDecodeException( "Unable to decode: " + 
					    fMathematicaSyntax );
	
	return tOMApplication;
    }


    /**
     * Decodes the Mathematica (Times) expression. <p>
     *
     * @since 1.0
     */

    public OMObject decodeTimes( String fMathematicaSyntax )
            throws  CodecDecodeException
        {
        // System.out.println( "Ons and Cvetan" );
       
        MathematicaCodec tParentCodec = (MathematicaCodec) mParent;
        Vector tVector = tParentCodec.parseArguments( fMathematicaSyntax );       
        int cnt = tVector.size()-1; 
        OMApplication tOMApplicationParent = new OMApplication();
        tOMApplicationParent.addElement( new OMSymbol( "arith1", "times" ) );
        OMObject tOMObject = tParentCodec.decodeOMObject(
                (String) tVector.elementAt( cnt ) );
        tOMApplicationParent.addElement( tOMObject );
        OMApplication tOMApplicationTmp = tOMApplicationParent;
        cnt--;
    	OMApplication tOMApplication = new OMApplication();
        
    	tOMApplication.addElement( new OMSymbol( "arith1", "times" ) );

    	if ( tVector.size() == 2 )
    	{
    	     tOMObject = 
    		tParentCodec.decodeOMObject( 
                        (String) tVector.elementAt( 0 ) );
     
    	    tOMApplication.addElement( tOMObject );

    	    tOMObject = tParentCodec.decodeOMObject(
    		    (String) tVector.elementAt( 1 ) );		
    	    
    	    tOMApplication.addElement( tOMObject );
    	    return tOMApplication ;
    	}
    	
        while (cnt > 0) {           
            OMApplication tOMApplicationChild = new OMApplication();;
            tOMApplicationChild.addElement( new OMSymbol( "arith1", "times" ) );
            tOMObject = tParentCodec.decodeOMObject(
                                (String) tVector.elementAt( cnt ) );
            tOMApplicationChild.addElement( tOMObject );
            cnt--;                 
            if(cnt == 0) {
                tOMObject = tParentCodec.decodeOMObject(
                        (String) tVector.elementAt( cnt ) );
                tOMApplicationChild.addElement( tOMObject );
                tOMApplicationTmp.addElement( tOMApplicationChild );
                return tOMApplicationParent;
            }
            tOMApplicationTmp.addElement( tOMApplicationChild );       
            tOMApplicationTmp = tOMApplicationChild;
        }                   
        return tOMApplicationParent;
        
}

    /**
     * Decodes the Mathematica (Minus) expression. <p>
     *
     * @since 1.0
     */

    public OMObject decodeUnaryMinus( String fMathematicaSyntax )
        throws CodecDecodeException
    {
	// System.out.println( "arith1Codec.decodeUnaryMinus" );

	OMApplication tOMApplication = new OMApplication();
       
	tOMApplication.addElement( new OMSymbol( "arith1", "unary_minus" ) );
	
	MathematicaCodec tParentCodec = (MathematicaCodec) mParent;
	Vector tVector = tParentCodec.parseArguments( fMathematicaSyntax );

	if ( tVector.size() == 1 )
	{
	    OMObject tOMObject = 
		tParentCodec.decodeOMObject( 
                    (String) tVector.elementAt( 0 ) );
 
	    tOMApplication.addElement( tOMObject );
	}
	else
	    throw new CodecDecodeException( "Unable to decode: " + 
					    fMathematicaSyntax );
	
	return tOMApplication;
    }
    
    
    /**
     * This is an auxiliar function that test if an OpenMath Object 
     * is of type Matrix or Vector. <p>
     *
     * @since 1.0
     */
    
    private boolean IsMatrixOrVector( OMObject fOMObject )
    {
        if ( fOMObject instanceof OMApplication )
        {        
            OMApplication tApplication = (OMApplication) fOMObject;
            
            if ( tApplication.getElementAt( 0 ) instanceof OMSymbol )
            {
                OMSymbol tSymbol = (OMSymbol) tApplication.getElementAt( 0 );
                
                if ( tSymbol.getCD().equals("linalg2") &&
                     tSymbol.getName().equals("matrix") )
                {
                    return true;
                }
                
                if ( tSymbol.getCD().equals("linalg2") &&
                     tSymbol.getName().equals("vector") )
                {
                    return true;
                }
            }
        }
        
        return false;
    }
}
