/*
 * $Id: complex1Codec.java,v 1.5 2002/11/20 23:51:51 mriem Exp $
 *
 * Copyright (c) 1999-2002, RIACA, Technische Universiteit Eindhoven (TU/e).
 * All Rights Reserved.
 *
 * The contents of this file are subject to the RIACA Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.riaca.win.tue.nl/
 */


package mathematicaCodec;
import nl.tue.win.riaca.openmath.codec.*;
import nl.tue.win.riaca.openmath.lang.*;
import java.util.*;


/**
 * A CD-codec that handles the OpenMath 'complex1' CD. <p>
 *
 * @author Manfred Riem (mriem@win.tue.nl)
 * @version 1.0
 */

public class complex1Codec
    extends CDCodec
{
    /**
     * Stores the names in this codec. <p>
     *
     * @since 1.0
     */
   
    public String[] sNames = 
    {
	"argument",
	"complex_cartesian",
	"complex_polar",
	"conjugate",
	"imaginary",
	"real"
    };


    /**
     * Stores the Mathematica equivalents to the OpenMath symbols. <p>
     *
     * @since 1.0
     */

    public String[] sMathNames =
    {
	"Arg",
	"Complex",
	"Conjugate",
	"Im",
	"Re"
    };


    /**
     * Constructor. <p>
     *
     * @since 1.0
     */

    public complex1Codec()
    {
	super();

	// System.out.println( "complex1Codec.<init>" );
    }

 
    /**
     * Constructor. <p>
     *
     * @since 1.0
     */
   
    public complex1Codec( Codec fParent )
    {
	super();
	
	mParent = fParent;
	
	// System.out.println( "complex1Codec.<init>" );
    }
  
  
    /**
     * Encodes the given OMObject to Mathematica-syntax. <p>
     *
     * @since 1.0
     */
   
    public String encodeOMObject( OMObject fOMObject )
        throws  CodecEncodeException
    {
	// System.out.println( "complex1Codec.encode" );
	
	OMApplication tApplication = (OMApplication) fOMObject;
	OMObject      tOMObject    = tApplication.getElementAt( 0 );
	
	if ( tOMObject instanceof OMSymbol )
	{
	    OMSymbol tOMSymbol = (OMSymbol) tOMObject;
	    
	    switch( getIndex( tOMSymbol.getName() ) )
	    {
	        case 0 : return encodeArgument( fOMObject );
	        case 1 : return encodeComplexCartesian( fOMObject );
	        case 2 : return encodeComplexPolar( fOMObject );
	        case 3 : return encodeConjugate( fOMObject );
	        case 4 : return encodeImaginary( fOMObject );
	        case 5 : return encodeReal( fOMObject );
	    }
	}
	
        throw new CodecEncodeException( "Unable to encode: " + fOMObject );
    }
    
    
    /**
     * Decodes the given Mathematica-syntax to a OMObject. <p>
     *
     * @since 1.0
     */
  
    public OMObject decodeOMObject( String fMathematicaSyntax )
        throws  CodecDecodeException
    {
	// System.out.println( "complex1Codec.decode" );
    
	String tOperator = 
	    fMathematicaSyntax.substring( 
                0, fMathematicaSyntax.indexOf( '[' ) );

	String tArguments =
	    fMathematicaSyntax.substring( fMathematicaSyntax.indexOf( '[' ) );

	if ( tOperator != null )
	{
	    tOperator = tOperator.trim();

	    switch( getMathIndex( tOperator ) )
	    {
	        case 0 :
		    return decodeArg( tArguments );
		case 1 :
		    return decodeComplex( tArguments );
      	        case 2 :
		    return decodeConjugate( tArguments );
	        case 3: 
		    return decodeIm( tArguments );
	        case 4 :
		    return decodeRe( tArguments );

	        default :
		    break;
	    }
	}
        
        throw new CodecDecodeException( 
            "Unable to decode: " + fMathematicaSyntax );
    }
  
  
    /**
     * Gets the index for the name. <p>
     *
     * @since 1.0
     */
   
    private int getIndex( String fName )
    {
	// System.out.println( "complex1Codec.getIndex" );
	
	for( int i=0; i<sNames.length; i++ )
	{
	    if ( fName.equals( sNames[i] ) )
		return i;
	}
    
	return -1;
    }


    /**
     * Gets the index for the Mathematica operator. <p>
     *
     * @since 1.0
     */
   
    private int getMathIndex( String fOperator )
    {
	// System.out.println( "complex1Codec.getMathIndex" );
	
	for( int i=0; i<sNames.length; i++ )
	{
	    if ( fOperator.equals( sMathNames[i] ) )
		return i;
	}
    
	return -1;
    }
  

    /**
     * Encodes the complex1.argument.
     *
     * @since 1.0
     */
   
    public String encodeArgument( OMObject fOMObject )
        throws  CodecEncodeException
    {
	// System.out.println( "complex1Codec.encodeArgument" );

	StringBuffer  tResult      = new StringBuffer();
	OMApplication tApplication = (OMApplication) fOMObject;
    
	tResult.append( "Arg[" );

	if ( tApplication.getElementAt( 1 ) != null )
	{
    	    tResult.append( 
		mParent.encodeOMObject( tApplication.getElementAt( 1 ) ) );
	} 
	else
	    throw new CodecEncodeException( "Unable to encode: " + fOMObject );
   
        tResult.append( "]" );
	return tResult.toString();
    }


    /**
     * Encodes the complex1.complex_cartesian. 
     *
     * @since 1.0
     */
   
    public String encodeComplexCartesian( OMObject fOMObject )
        throws  CodecEncodeException
    {
	// System.out.println( "complex1Codec.encodeComplexCartesian" );
     
	StringBuffer  tResult      = new StringBuffer();
	OMApplication tApplication = (OMApplication) fOMObject;
    
	tResult.append( "Complex[" );
	
	if ( tApplication.getElementAt( 1 ) != null )
	{
	    tResult.append(
		 mParent.encodeOMObject( tApplication.getElementAt( 1 ) ) );
	}
	else
	    throw new CodecEncodeException( "Unable to encode: " + fOMObject );

	tResult.append( "," );

        if ( tApplication.getElementAt( 2 ) != null )
        {
	    tResult.append(
		 mParent.encodeOMObject( tApplication.getElementAt( 2 ) ) );
	}
	else
	    throw new CodecEncodeException( "Unable to encode: " + fOMObject );

	tResult.append( "]" );
	return tResult.toString();
    }


    /**
     * Encodes the complex1.complex_polar.
     *
     * @since 1.0
     */
   
    public String encodeComplexPolar( OMObject fOMObject )
        throws  CodecEncodeException
    {
	// System.out.println( "complex1Codec.encodeComplexPolar" );
     
	StringBuffer  tResult      = new StringBuffer();
	OMApplication tApplication = (OMApplication) fOMObject;
	int           tIndex       = 1;
    
	tResult.append( "Times[" );
	
        if ( tApplication.getElementAt( 1 ) != null )
            tResult.append( 
                mParent.encodeOMObject( tApplication.getElementAt( 1 ) ) );
        else
	    throw new CodecEncodeException( "Unable to encode: " + fOMObject );
      
        tResult.append( "," );
        tResult.append( "Power[E,Times[" );
        
        if ( tApplication.getElementAt( 2 ) != null )
            tResult.append( 
                mParent.encodeOMObject( tApplication.getElementAt( 2 ) ) );
        else
	    throw new CodecEncodeException( "Unable to encode: " + fOMObject );
        
        tResult.append( ",Complex[0,1]]]]" );
	return tResult.toString();
    }


    /**
     * Encodes the complex1.conjugate.
     *
     * @since 1.0
     */
   
    public String encodeConjugate( OMObject fOMObject )
        throws  CodecEncodeException
    {
	// System.out.println( "complex1Codec.encodeConjugate" );
     
	StringBuffer  tResult      = new StringBuffer();
	OMApplication tApplication = (OMApplication) fOMObject;
    
	tResult.append( "Conjugate[" );

	if ( tApplication.getElementAt( 1 ) != null )
	{
    	    tResult.append( 
		mParent.encodeOMObject( tApplication.getElementAt( 1 ) ) );
	} 
	else
	    throw new CodecEncodeException( "Unable to encode: " + fOMObject );
   
        tResult.append( "]" );
	return tResult.toString();
    }


    /**
     * Encodes the complex1.imaginary.
     *
     * @since 1.0
     */
    
    public String encodeImaginary( OMObject fOMObject )
        throws  CodecEncodeException
    {
	// System.out.println( "complex1Codec.encodeImaginary" );
     
	StringBuffer  tResult      = new StringBuffer();
	OMApplication tApplication = (OMApplication) fOMObject;
    
	tResult.append( "Im[" );

	if ( tApplication.getElementAt( 1 ) != null )
	{
    	    tResult.append( 
		mParent.encodeOMObject( tApplication.getElementAt( 1 ) ) );
	} 
	else
	    throw new CodecEncodeException( "Unable to encode: " + fOMObject );
   
        tResult.append( "]" );
	return tResult.toString();
    }


    /**
     * Encodes the complex1.real.
     *
     * @since 1.0
     */
   
    public String encodeReal( OMObject fOMObject )
        throws  CodecEncodeException
    {
	// System.out.println( "complex1Codec.encodeReal" );
     
	StringBuffer  tResult      = new StringBuffer();
	OMApplication tApplication = (OMApplication) fOMObject;
    
	tResult.append( "Re[" );

	if ( tApplication.getElementAt( 1 ) != null )
	{
    	    tResult.append( 
		mParent.encodeOMObject( tApplication.getElementAt( 1 ) ) );
	} 
	else
	    throw new CodecEncodeException( "Unable to encode: " + fOMObject );
   
        tResult.append( "]" );
	return tResult.toString();
    }


    /**
     * Decodes the Mathematica (Arg) expression. <p>
     *
     * @since 1.0
     */

    public OMObject decodeArg( String fMathematicaSyntax )
        throws  CodecDecodeException
    {
	// System.out.println( "complex1Codec.decodeArg" );

	OMApplication tOMApplication = new OMApplication();
       
	tOMApplication.addElement( new OMSymbol( "complex1", "argument" ) );
	
	MathematicaCodec tParentCodec = (MathematicaCodec) mParent;
	Vector tVector = tParentCodec.parseArguments( fMathematicaSyntax );

	if ( tVector.size() == 1 )
	{
	    OMObject tOMObject = 
		tParentCodec.decodeOMObject( 
                    (String) tVector.elementAt( 0 ) );
 
	    tOMApplication.addElement( tOMObject );
	}
	else
	    throw new CodecDecodeException( "Unable to decode: " + fMathematicaSyntax );
   
	return tOMApplication;
    }


    /**
     * Decodes the Mathematica (Complex) expression. <p>
     *
     * @since 1.0
     */

    public OMObject decodeComplex( String fMathematicaSyntax )
        throws  CodecDecodeException
    {
	// System.out.println( "complex1Codec.decodeComplex" );

	OMApplication tOMApplication = new OMApplication();
       
	tOMApplication.addElement( 
            new OMSymbol( "complex1", "complex_cartesian" ) );
	
	MathematicaCodec tParentCodec = (MathematicaCodec) mParent;
	Vector tVector = tParentCodec.parseArguments( fMathematicaSyntax );

	if ( tVector.size() == 2 )
	{
	    OMObject tOMObject = 
		tParentCodec.decodeOMObject( 
                    (String) tVector.elementAt( 0 ) );
 
	    tOMApplication.addElement( tOMObject );

	    tOMObject = tParentCodec.decodeOMObject(
		    (String) tVector.elementAt( 1 ) );		
	    
	    tOMApplication.addElement( tOMObject );
	}
	else
	    throw new CodecDecodeException( "Unable to decode: " + fMathematicaSyntax );
   
	return tOMApplication;
    }


    /**
     * Decodes the Mathematica (Conjugate) expression. <p>
     *
     * @since 1.0
     */

    public OMObject decodeConjugate( String fMathematicaSyntax )
        throws  CodecDecodeException
    {
	// System.out.println( "complex1Codec.decodeConjugate" );

	OMApplication tOMApplication = new OMApplication();
       
	tOMApplication.addElement( new OMSymbol( "complex1", "conjugate" ) );
	
	MathematicaCodec tParentCodec = (MathematicaCodec) mParent;
	Vector tVector = tParentCodec.parseArguments( fMathematicaSyntax );

	if ( tVector.size() == 1 )
	{
	    OMObject tOMObject = 
		tParentCodec.decodeOMObject( 
                    (String) tVector.elementAt( 0 ) );
 
	    tOMApplication.addElement( tOMObject );
	}
	else
	    throw new CodecDecodeException( "Unable to decode: " + fMathematicaSyntax );
   
	return tOMApplication;
    }


    /**
     * Decodes the Mathematica (Im) expression. <p>
     *
     * @since 1.0
     */

    public OMObject decodeIm( String fMathematicaSyntax )
        throws  CodecDecodeException
    {
	// System.out.println( "complex1Codec.decodeIm" );

	OMApplication tOMApplication = new OMApplication();
       
	tOMApplication.addElement( new OMSymbol( "complex1", "imaginary" ) );
	
	MathematicaCodec tParentCodec = (MathematicaCodec) mParent;
	Vector tVector = tParentCodec.parseArguments( fMathematicaSyntax );

	if ( tVector.size() == 1 )
	{
	    OMObject tOMObject = 
		tParentCodec.decodeOMObject( 
                    (String) tVector.elementAt( 0 ) );
 
	    tOMApplication.addElement( tOMObject );
	}
	else
	    throw new CodecDecodeException( "Unable to decode: " + fMathematicaSyntax );
   
	return tOMApplication;    
    }


    /**
     * Decodes the Mathematica (Re) expression. <p>
     *
     * @since 1.0
     */

    public OMObject decodeRe( String fMathematicaSyntax )
        throws  CodecDecodeException
    {
	// System.out.println( "complex1Codec.decodeRe" );

	OMApplication tOMApplication = new OMApplication();
       
	tOMApplication.addElement( new OMSymbol( "complex1", "real" ) );
	
	MathematicaCodec tParentCodec = (MathematicaCodec) mParent;
	Vector tVector = tParentCodec.parseArguments( fMathematicaSyntax );

	if ( tVector.size() == 1 )
	{
	    OMObject tOMObject = 
		tParentCodec.decodeOMObject( 
                    (String) tVector.elementAt( 0 ) );
 
	    tOMApplication.addElement( tOMObject );
	}
	else
	    throw new CodecDecodeException( "Unable to decode: " + fMathematicaSyntax );
   
	return tOMApplication;    
    }
}
