/*
 * $Id: integer1Codec.java,v 1.6 2003/06/19 13:14:49 mriem Exp $
 *
 * Copyright (c) 1999-2002, RIACA, Technische Universiteit Eindhoven (TU/e).
 * All Rights Reserved.
 *
 * The contents of this file are subject to the RIACA Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.riaca.win.tue.nl/
 */


package mathematicaCodec;

import nl.tue.win.riaca.openmath.codec.*;
import nl.tue.win.riaca.openmath.lang.*;
import java.util.*;


/**
 * A CD-codec that handles the OpenMath 'integer1' CD. <p>
 *
 * @author Manfred Riem (mriem@win.tue.nl)
 * @version 1.0
 */

public class integer1Codec
    extends CDCodec
{
    /**
     * Stores the names in this codec. <p>
     *
     * @since 1.0
     */
   
    public String[] sNames = 
    {
    	"factorial",
    	"factorof",
    	"quotient",
    	"remainder"
    };


    /**
     * Stores the Mathematica equivalents to the OpenMath symbols. <p>
     *
     * @since 1.0
     */

    public String[] sMathNames =
    {
	"Factorial",
	"Quotient",
	"Mod"
    };


    /**
     * Constructor. <p>
     *
     * @since 1.0
     */

    public integer1Codec()
    {
	super();

	// System.out.println( "integer1Codec.<init>" );
    }

 
    /**
     * Constructor. <p>
     *
     * @since 1.0
     */
   
    public integer1Codec( Codec fParent )
    {
	super();
	
	mParent = fParent;
	
	// System.out.println( "integer1Codec.<init>" );
    }
  
  
    /**
     * Encodes the given OMObject to Mathematica-syntax. <p>
     *
     * @since 1.0
     */
   
    public String encodeOMObject( OMObject fOMObject )
        throws  CodecEncodeException
    {
	// System.out.println( "integer1Codec.encode" );
	
	OMApplication tApplication = (OMApplication) fOMObject;
	OMObject      tOMObject    = tApplication.getElementAt( 0 );
	
	if ( tOMObject instanceof OMSymbol )
	{
	    OMSymbol tOMSymbol = (OMSymbol) tOMObject;
	    
	    switch( getIndex( tOMSymbol.getName() ) )
	    {
	        case 0 :
		    return encodeFactorial( fOMObject );
	        case 1 :
		    return encodeFactorOf( fOMObject );
	        case 2 :
		    return encodeQuotient( fOMObject );
	        case 3: 
		    return encodeRemainder( fOMObject );
	    }
	}
    
        throw new CodecEncodeException( 
	    "Unable to encode: " + fOMObject );
    }


    /**
     * Decodes the given Mathematica-syntax to a OMObject. <p>
     *
     * @since 1.0
     */
  
    public OMObject decodeOMObject( String fMathematicaSyntax )
        throws  CodecDecodeException
    {
	// System.out.println( "integer1Codec.decode" );
    
	String tOperator = 
	    fMathematicaSyntax.substring( 
                0, fMathematicaSyntax.indexOf( '[' ) );

	String tArguments =
	    fMathematicaSyntax.substring( fMathematicaSyntax.indexOf( '[' ) );

	if ( tOperator != null )
	{
	    tOperator = tOperator.trim();

	    switch( getMathIndex( tOperator ) )
	    {
	        case 0 :
		    return decodeFactorial( tArguments );
		case 1 :
		    return decodeQuotient( tArguments );
      	        case 2 :
		    return decodeMod( tArguments );
		    
	        default :
		    break;
	    }
	}
	
        throw new CodecDecodeException( 
            "Unable to decode: " + fMathematicaSyntax );
    }
  
  
    /**
     * Gets the index for the name. <p>
     *
     * @since 1.0
     */
   
    private int getIndex( String fName )
    {
	// System.out.println( "integer1Codec.getIndex" );
	
	for( int i=0; i<sNames.length; i++ )
	{
	    if ( fName.equals( sNames[i] ) )
		return i;
	}
    
	return -1;
    }


    /**
     * Gets the index for the Mathematica operator. <p>
     *
     * @since 1.0
     */
   
    private int getMathIndex( String fOperator )
    {
	// System.out.println( "integer1Codec.getMathIndex" );
	
	for( int i=0; i<sNames.length; i++ )
	{
	    if ( fOperator.equals( sMathNames[i] ) )
		return i;
	}
    
	return -1;
    }
  

    /**
     * Encodes the integer1.factorial.
     *
     * @since 1.0
     */
   
    public String encodeFactorial( OMObject fOMObject )
        throws  CodecEncodeException
    {
	// System.out.println( "integer1Codec.encodeFactorial" );

	StringBuffer  tResult      = new StringBuffer();
	OMApplication tApplication = (OMApplication) fOMObject;
    
	tResult.append( "Factorial[" );

	if ( tApplication.getElementAt( 1 ) != null )
	{
    	    tResult.append( 
		mParent.encodeOMObject( tApplication.getElementAt( 1 ) ) );
	} 
	else
            throw new CodecEncodeException( "Unable to encode: " + fOMObject );
   
        tResult.append( "]" );
	return tResult.toString();
    }
    
    /**
     * Encodes the integer1.factorof. 
     *
     * @since 1.0
     */
    
    public String encodeFactorOf( OMObject fOMObject )
        throws CodecEncodeException {
	
	// System.out.println( "integer1Codec.encodeFactorOf" );
	
	StringBuffer  tResult      = new StringBuffer();
	OMApplication tApplication = (OMApplication) fOMObject;
	
	tResult.append( "Equal[Mod[" );
	
	if ( tApplication.getElementAt( 2 ) != null ) {
	    tResult.append(
		 mParent.encodeOMObject( tApplication.getElementAt( 2 ) ) );
	}
	else
            throw new CodecEncodeException( "Unable to encode: " + fOMObject );
	
	tResult.append( "," );
	
        if ( tApplication.getElementAt( 1 ) != null ) {
	    tResult.append(
		 mParent.encodeOMObject( tApplication.getElementAt( 1 ) ) );
	}
	else
            throw new CodecEncodeException( "Unable to encode: " + fOMObject );
	
	tResult.append( "],0]" );
	return tResult.toString();
    }
    
    /**
     * Encodes the integer1.quotient 
     *
     * @since 1.0
     */
   
    public String encodeQuotient( OMObject fOMObject )
        throws  CodecEncodeException
    {
	// System.out.println( "integer1Codec.encodeQuotient" );
     
	StringBuffer  tResult      = new StringBuffer();
	OMApplication tApplication = (OMApplication) fOMObject;
    
	tResult.append( "Quotient[" );
	
	if ( tApplication.getElementAt( 1 ) != null )
	{
	    tResult.append(
		 mParent.encodeOMObject( tApplication.getElementAt( 1 ) ) );
	}
	else
            throw new CodecEncodeException( "Unable to encode: " + fOMObject );

	tResult.append( "," );

        if ( tApplication.getElementAt( 2 ) != null )
        {
	    tResult.append(
		 mParent.encodeOMObject( tApplication.getElementAt( 2 ) ) );
	}
	else
            throw new CodecEncodeException( "Unable to encode: " + fOMObject );

	tResult.append( "]" );
	return tResult.toString();
    }


    /**
     * Encodes the integer1.remainder
     *
     * @since 1.0
     */
   
    public String encodeRemainder( OMObject fOMObject )
        throws  CodecEncodeException
    {
	// System.out.println( "integer1Codec.encodeRemainder" );
     
	StringBuffer  tResult      = new StringBuffer();
	OMApplication tApplication = (OMApplication) fOMObject;
    
	tResult.append( "Mod[" );
	
	if ( tApplication.getElementAt( 1 ) != null )
	{
	    tResult.append(
		 mParent.encodeOMObject( tApplication.getElementAt( 1 ) ) );
	}
	else
            throw new CodecEncodeException( "Unable to encode: " + fOMObject );

	tResult.append( "," );

        if ( tApplication.getElementAt( 2 ) != null )
        {
	    tResult.append(
		 mParent.encodeOMObject( tApplication.getElementAt( 2 ) ) );
	}
	else
            throw new CodecEncodeException( "Unable to encode: " + fOMObject );

	tResult.append( "]" );
	return tResult.toString();
    }


    /**
     * Decodes the Mathematica (Factorial) expression. <p>
     *
     * @since 1.0
     */

    public OMObject decodeFactorial( String fMathematicaSyntax )
        throws  CodecDecodeException
    {
	// System.out.println( "integer1Codec.decodeFactorial" );

	OMApplication tOMApplication = new OMApplication();
       
	tOMApplication.addElement( new OMSymbol( "integer1", "factorial" ) );
	
	MathematicaCodec tParentCodec = (MathematicaCodec) mParent;
	Vector tVector = tParentCodec.parseArguments( fMathematicaSyntax );

	if ( tVector.size() == 1 )
	{
	    OMObject tOMObject = 
		tParentCodec.decodeOMObject( 
                    (String) tVector.elementAt( 0 ) );
 
	    tOMApplication.addElement( tOMObject );
	}
	else
            throw new CodecDecodeException( 
                "Unable to decode: " + fMathematicaSyntax );
   
	return tOMApplication;
    }


    /**
     * Decodes the Mathematica (Quotient) expression. <p>
     *
     * @since 1.0
     */

    public OMObject decodeQuotient( String fMathematicaSyntax )
        throws  CodecDecodeException
    {
	// System.out.println( "integer1Codec.decodeQuotient" );

	OMApplication tOMApplication = new OMApplication();
       
	tOMApplication.addElement( new OMSymbol( "integer1", "quotient" ) );
	
	MathematicaCodec tParentCodec = (MathematicaCodec) mParent;
	Vector tVector = tParentCodec.parseArguments( fMathematicaSyntax );

	if ( tVector.size() == 2 )
	{
	    OMObject tOMObject = 
		tParentCodec.decodeOMObject( 
                    (String) tVector.elementAt( 0 ) );
	    
	    tOMApplication.addElement( tOMObject );
	    
	    tOMObject = tParentCodec.decodeOMObject(
	        (String) tVector.elementAt( 1 ) );		
	    
	    tOMApplication.addElement( tOMObject );
	}
	else
            throw new CodecDecodeException( 
                "Unable to decode: " + fMathematicaSyntax );
   
	return tOMApplication;
    }


    /**
     * Decodes the Mathematica (Mod) expression. <p>
     *
     * @since 1.0
     */

    public OMObject decodeMod( String fMathematicaSyntax )
        throws  CodecDecodeException
    {
	// System.out.println( "integer1Codec.decodeMod" );

	OMApplication tOMApplication = new OMApplication();
       
	tOMApplication.addElement( new OMSymbol( "integer1", "remainder" ) );
	
	MathematicaCodec tParentCodec = (MathematicaCodec) mParent;
	Vector tVector = tParentCodec.parseArguments( fMathematicaSyntax );

	if ( tVector.size() == 2 )
	{
	    OMObject tOMObject = 
		tParentCodec.decodeOMObject( 
                    (String) tVector.elementAt( 0 ) );
 
	    tOMApplication.addElement( tOMObject );

	    tOMObject = tParentCodec.decodeOMObject(
		    (String) tVector.elementAt( 1 ) );		
	    
	    tOMApplication.addElement( tOMObject );
	}
	else
            throw new CodecDecodeException( 
                "Unable to decode: " + fMathematicaSyntax );
   
	return tOMApplication;
    }
}
