/*
 * $Id: linalg1Codec.java,v 1.4 2002/11/20 23:51:51 mriem Exp $
 *
 * Copyright (c) 1999-2002, RIACA, Technische Universiteit Eindhoven (TU/e).
 * All Rights Reserved.
 *
 * The contents of this file are subject to the RIACA Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.riaca.win.tue.nl/
 */


package mathematicaCodec;

import nl.tue.win.riaca.openmath.codec.*;
import nl.tue.win.riaca.openmath.lang.*;

import java.util.*;


/**
 * A CD-codec that handles the OpenMath 'linalg1' CD. <p>
 *
 * @author Manfred Riem (mriem@win.tue.nl)
 * @version 1.0
 */

public class linalg1Codec
    extends CDCodec
{
    /**
     * Stores the names in this codec. <p>
     *
     * @since 1.0
     */
   
    public String[] sNames = 
    {
	"determinant",
	"matrix_selector",
	"outerproduct",
	"scalarproduct",
	"transpose",
	"vector_selector",
	"vectorproduct",
	"inverse"
    };


    /**
     * Stores the Mathematica equivalents to the OpenMath symbols. <p>
     *
     * @since 1.0
     */

    public String[] sMathNames =
    {
        "Det",
        "Cross",
        "Inverse"
    	
    };

    
    /**
     * Constructor. <p>
     *
     * @since 1.0
     */

    public linalg1Codec()
    {
	super();

	// System.out.println( "linalg1Codec.<init>" );
    }

 
    /**
     * Constructor. <p>
     *
     * @since 1.0
     */
   
    public linalg1Codec( Codec fParent )
    {
	super();
	
	mParent = fParent;
	
	// System.out.println( "linalg1Codec.<init>" );
    }
  
  
    /**
     * Encodes the given OMObject to Mathematica-syntax. <p>
     *
     * @since 1.0
     */
   
    public String encodeOMObject( OMObject fOMObject )
        throws  CodecEncodeException
    {
	// System.out.println( "linalg1Codec.encodeOMObject" );
	
	OMApplication tApplication = (OMApplication) fOMObject;
	OMObject      tOMObject    = tApplication.getElementAt( 0 );
	
	if ( tOMObject instanceof OMSymbol )
	{
	    OMSymbol tOMSymbol = (OMSymbol) tOMObject;
	    
	    switch( getIndex( tOMSymbol.getName() ) )
	    {
	        case 0 :
		    return encodeDeterminant( fOMObject );
                    
                case 1 :
                    return encodeMatrixSelector( fOMObject );
                    
                case 2 :
                    return encodeOuterProduct( fOMObject );
                    
                case 3 :
                    return encodeScalarProduct( fOMObject );
                    
                case 4 :
                    return encodeTranspose( fOMObject );
                    
                case 5 :
                    return encodeVectorSelector( fOMObject );
                    
                case 6 :
                    return encodeVectorProduct( fOMObject );
                case 7 :
                    return encodeInverse( fOMObject );
	    }
	}
    
        throw new CodecEncodeException( 
            "Unable to encode: " + fOMObject );
    }


    /**
     * Decodes the given Mathematica-syntax to a OMObject. <p>
     *
     * @since 1.0
     */
  
    public OMObject decodeOMObject( String fMathematicaSyntax )
        throws  CodecDecodeException
    {
	// System.out.println( "linalg1Codec.decodeOMObject" );
       
	String tOperator = 
	    fMathematicaSyntax.substring( 
                0, fMathematicaSyntax.indexOf( '[' ) );

	String tArguments =
	    fMathematicaSyntax.substring( fMathematicaSyntax.indexOf( '[' ) );

	if ( tOperator != null )
	{
	    tOperator = tOperator.trim();

	    switch( getMathIndex( tOperator ) )
	    {
	        case 0  : return decodeDet( tArguments );
	        case 1  : return decodeCross( tArguments );
	        default : break;
	    }
	}
    
        throw new CodecDecodeException( 
            "Unable to decode: " + fMathematicaSyntax  );
    }
    
    
    /**
     * Gets the index for the name. <p>
     *
     * @since 1.0
     */
    
    private int getIndex( String fName )
    {
	// System.out.println( "linalg1Codec.getIndex" );
	
	for( int i=0; i<sNames.length; i++ )
	{
	    if ( fName.equals( sNames[i] ) )
		return i;
	}
    
	return -1;
    }
 

    /**
     * Gets the index for the Mathematica operator. <p>
     *
     * @since 1.0
     */
   
    private int getMathIndex( String fOperator )
    {
	// System.out.println( "integer1Codec.getMathIndex" );
	
	for( int i=0; i<sNames.length; i++ )
	{
	    if ( fOperator.equals( sMathNames[i] ) )
		return i;
	}
    
	return -1;
    }
    

    /**
     * Encodes the linalg1.inverse. <p>
     *
     * @since 1.0
     */
   
    public String encodeInverse( OMObject fOMObject )
        throws  CodecEncodeException
    {
	// System.out.println( "linalg1Codec.encodeDeterminant" );
    
	StringBuffer  tResult      = new StringBuffer();
	OMApplication tApplication = (OMApplication) fOMObject;
	
	tResult.append( "Inverse[" );
	
	if ( tApplication.getElementAt( 1 ) != null )
	{
	    tResult.append(
		 mParent.encodeOMObject( tApplication.getElementAt( 1 ) ) );
	}
	else
	    throw new CodecEncodeException( 
                "Unable to encode: " + fOMObject );

	tResult.append( "]" );
	return tResult.toString();
    }
    

    /**
     * Encodes the linalg1.determinant. <p>
     *
     * @since 1.0
     */
   
    public String encodeDeterminant( OMObject fOMObject )
        throws  CodecEncodeException
    {
	// System.out.println( "linalg1Codec.encodeDeterminant" );
    
	StringBuffer  tResult      = new StringBuffer();
	OMApplication tApplication = (OMApplication) fOMObject;
	
	tResult.append( "Det[" );
	
	if ( tApplication.getElementAt( 1 ) != null )
	{
	    tResult.append(
		 mParent.encodeOMObject( tApplication.getElementAt( 1 ) ) );
	}
	else
	    throw new CodecEncodeException( 
                "Unable to encode: " + fOMObject );

	tResult.append( "]" );
	return tResult.toString();
    }


    /**
     * Encodes the linalg1.matrix_selector. <p>
     *
     * @since 1.0
     */
   
    public String encodeMatrixSelector( OMObject fOMObject )
        throws  CodecEncodeException
    {
	// System.out.println( "linalg1Codec.encodeMatrixSelector" );
    
	StringBuffer  tResult      = new StringBuffer();
	OMApplication tApplication = (OMApplication) fOMObject;
    
	tResult.append( "Part[" );
	
	if ( tApplication.getElementAt( 3 ) != null )
	{
	    tResult.append(
		 mParent.encodeOMObject( tApplication.getElementAt( 3 ) ) );
	}
	else
	    throw new CodecEncodeException( 
                "Unable to encode: " + fOMObject );

        tResult.append( "," );

        if ( tApplication.getElementAt( 1 ) != null )
	{
	    tResult.append(
		 mParent.encodeOMObject( tApplication.getElementAt( 1 ) ) );
	}
	else
	    throw new CodecEncodeException( 
                "Unable to encode: " + fOMObject );

        tResult.append( "," );
        
        if ( tApplication.getElementAt( 2 ) != null )
	{
	    tResult.append(
		 mParent.encodeOMObject( tApplication.getElementAt( 2 ) ) );
	}
	else
	    throw new CodecEncodeException( 
                "Unable to encode: " + fOMObject );

        tResult.append( "]" );
	return tResult.toString();
    }


    /**
     * Encodes the linalg1.outerproduct. <p>
     *
     * @since 1.0
     */
   
    public String encodeOuterProduct( OMObject fOMObject )
        throws  CodecEncodeException
    {
	// System.out.println( "linalg1Codec.encodeOuterProduct" );
    
	StringBuffer  tResult      = new StringBuffer();
	OMApplication tApplication = (OMApplication) fOMObject;
    
	tResult.append( "Outer[Times[" );
	
        if ( tApplication.getElementAt( 1 ) != null )
	{
	    tResult.append(
		 mParent.encodeOMObject( tApplication.getElementAt( 1 ) ) );
	}
	else
	    throw new CodecEncodeException( 
                "Unable to encode: " + fOMObject );

        tResult.append( "," );
        
        if ( tApplication.getElementAt( 2 ) != null )
	{
	    tResult.append(
		 mParent.encodeOMObject( tApplication.getElementAt( 2 ) ) );
	}
	else
	    throw new CodecEncodeException( 
                "Unable to encode: " + fOMObject );

        tResult.append( "]]" );
	return tResult.toString();
    }

    
    /**
     * Encodes the linalg1.scalarproduct. <p>
     *
     * @since 1.0
     */
   
    public String encodeScalarProduct( OMObject fOMObject )
        throws  CodecEncodeException
    {
	// System.out.println( "linalg1Codec.encodeScalarProduct" );
    
	StringBuffer  tResult      = new StringBuffer();
	OMApplication tApplication = (OMApplication) fOMObject;
    
	tResult.append( "Dot[" );
	
        if ( tApplication.getElementAt( 1 ) != null )
	{
	    tResult.append(
		 mParent.encodeOMObject( tApplication.getElementAt( 1 ) ) );
	}
	else
	    throw new CodecEncodeException( 
                "Unable to encode: " + fOMObject );

        tResult.append( "," );
        
        if ( tApplication.getElementAt( 2 ) != null )
	{
	    tResult.append(
		 mParent.encodeOMObject( tApplication.getElementAt( 2 ) ) );
	}
	else
	    throw new CodecEncodeException( 
                "Unable to encode: " + fOMObject );

        tResult.append( "]" );
	return tResult.toString();
    }

    
    /**
     * Encodes the linalg1.transpose. <p>
     *
     * @since 1.0
     */
   
    public String encodeTranspose( OMObject fOMObject )
        throws  CodecEncodeException
    {
	// System.out.println( "linalg1Codec.encodeTranspose" );
    
	StringBuffer  tResult      = new StringBuffer();
	OMApplication tApplication = (OMApplication) fOMObject;
    
	tResult.append( "Transpose[" );
	
        if ( tApplication.getElementAt( 1 ) != null )
	{
	    tResult.append(
		 mParent.encodeOMObject( tApplication.getElementAt( 1 ) ) );
	}
	else
	    throw new CodecEncodeException( 
                "Unable to encode: " + fOMObject );

        tResult.append( "]" );
	return tResult.toString();
    }

    
    /**
     * Encodes the linalg1.vector_selector. <p>
     *
     * @since 1.0
     */
   
    public String encodeVectorSelector( OMObject fOMObject )
        throws  CodecEncodeException
    {
	// System.out.println( "linalg1Codec.encodeVectorSelector" );
    
	StringBuffer  tResult      = new StringBuffer();
	OMApplication tApplication = (OMApplication) fOMObject;
    
	tResult.append( "Part[" );
	
        if ( tApplication.getElementAt( 2 ) != null )
	{
	    tResult.append(
		 mParent.encodeOMObject( tApplication.getElementAt( 2 ) ) );
	}
	else
	    throw new CodecEncodeException( 
                "Unable to encode: " + fOMObject );

        tResult.append( "," );
        
        if ( tApplication.getElementAt( 1 ) != null )
	{
	    tResult.append(
		 mParent.encodeOMObject( tApplication.getElementAt( 1 ) ) );
	}
	else
	    throw new CodecEncodeException( 
                "Unable to encode: " + fOMObject );

        tResult.append( "]" );
	return tResult.toString();
    }

    
    /**
     * Encodes the linalg1.vectorproduct. <p>
     *
     * @since 1.0
     */
   
    public String encodeVectorProduct( OMObject fOMObject )
        throws  CodecEncodeException
    {
	// System.out.println( "linalg1Codec.encodeVectorProduct" );
    
	StringBuffer  tResult      = new StringBuffer();
	OMApplication tApplication = (OMApplication) fOMObject;
    
	tResult.append( "Cross[" );
	
        if ( tApplication.getElementAt( 1 ) != null )
	{
	    tResult.append(
		 mParent.encodeOMObject( tApplication.getElementAt( 1 ) ) );
	}
	else
	    throw new CodecEncodeException( 
                "Unable to encode: " + fOMObject );

        tResult.append( "," );
        
        if ( tApplication.getElementAt( 2 ) != null )
	{
	    tResult.append(
		 mParent.encodeOMObject( tApplication.getElementAt( 2 ) ) );
	}
	else
	    throw new CodecEncodeException( 
                "Unable to encode: " + fOMObject );

        tResult.append( "]" );
	return tResult.toString();
    }

    
    /**
     * Decodes the Mathematica (Det) expression. <p>
     *
     * @since 1.0
     */

    public OMObject decodeDet( String fMathematicaSyntax )
        throws  CodecDecodeException
    {
	// System.out.println( "integer1Codec.decodeDet" );

	OMApplication tOMApplication = new OMApplication();
       
	tOMApplication.addElement( new OMSymbol( "linalg1", "determinant" ) );
	
	MathematicaCodec tParentCodec = (MathematicaCodec) mParent;
	Vector tVector = tParentCodec.parseArguments( fMathematicaSyntax );

	if ( tVector.size() == 1 )
	{
	    OMObject tOMObject = 
		tParentCodec.decodeOMObject( 
                    (String) tVector.elementAt( 0 ) );
 
	    tOMApplication.addElement( tOMObject );
	}
	else
            throw new CodecDecodeException( 
                "Unable to decode: " + fMathematicaSyntax );
   
	return tOMApplication;
    }
    
    /**
     * Decodes the Mathematica (Outer) expression. <p>
     *
     * @since 1.0
     */

    public OMObject decodeOuter( String fMathematicaSyntax )
        throws  CodecDecodeException
    {
	// System.out.println( "integer1Codec.decodeDet" );

	OMApplication tOMApplication = new OMApplication();
       
	tOMApplication.addElement( new OMSymbol( "linalg1", "Outer" ) );
	
	MathematicaCodec tParentCodec = (MathematicaCodec) mParent;
	Vector tVector = tParentCodec.parseArguments( fMathematicaSyntax );

	if ( tVector.size() == 1 )
	{
	    OMObject tOMObject = 
		tParentCodec.decodeOMObject( 
                    (String) tVector.elementAt( 0 ) );
 
	    tOMApplication.addElement( tOMObject );
	}
	else
            throw new CodecDecodeException( 
                "Unable to decode: " + fMathematicaSyntax );
   
	return tOMApplication;
    }
    
    
    
    /**
     * Decodes the Mathematica (Cross) expression. <p>
     *
     * @since 1.0
     */

    public OMObject decodeCross( String fMathematicaSyntax )
        throws  CodecDecodeException
    {
	// System.out.println( "arith1Codec.decodeGcd" );
	
	OMApplication tOMApplication = new OMApplication();
	
	tOMApplication.addElement( new OMSymbol( "linalg1", "vectorproduct" ) );
	
	MathematicaCodec tParentCodec = (MathematicaCodec) mParent;
	Vector tVector = tParentCodec.parseArguments( fMathematicaSyntax );
	Enumeration tEnum = tVector.elements();
	
	for( ; tEnum.hasMoreElements(); )
	{
	    OMObject tOMObject = 
		tParentCodec.decodeOMObject( 
                    (String) tEnum.nextElement() );
	    
	    if ( tOMObject != null )
		tOMApplication.addElement( tOMObject );
	    else
		throw new CodecDecodeException( "Unable to decode: " + 
						fMathematicaSyntax );
	}
	
	return tOMApplication;
    }
}
