/*
 * $Id: list1Codec.java,v 1.4 2002/11/20 23:51:51 mriem Exp $
 *
 * Copyright (c) 1999-2002, RIACA, Technische Universiteit Eindhoven (TU/e).
 * All Rights Reserved.
 *
 * The contents of this file are subject to the RIACA Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.riaca.win.tue.nl/
 */


package mathematicaCodec;

import nl.tue.win.riaca.openmath.codec.*;
import nl.tue.win.riaca.openmath.lang.*;
import java.util.*;


/**
 * A CD-codec that handles the OpenMath 'list1'-cd. <p>
 *
 * @author Manfred Riem (mriem@win.tue.nl)
 * @version 1.0
 */

public class list1Codec
    extends CDCodec
{
    /**
     * Stores the names in this codec. <p>
     *
     * @since 1.0
     */
   
    public String[] sNames = 
    {
    	"list",
    	"map",
	"such_that"
    };


    /**
     * Stores the Mathematica equivalents to the OpenMath symbols. <p>
     *
     * @since 1.0
     */

    public String[] sMathNames =
    {
    	"List",
    };
    
    
    /**
     * Constructor. <p>
     *
     * @since 1.0
     */

    public list1Codec()
    {
    	super();
        
    	// System.out.println( "list1Codec.<init>" );
    }

 
    /**
     * Constructor. <p>
     *
     * @since 1.0
     */
   
    public list1Codec( Codec fParent )
    {
    	super();
	    
    	mParent = fParent;
	
	// System.out.println( "list1Codec.<init>" );
    }
  
  
    /**
     * Encodes the given OMObject to Mathematica-syntax. <p>
     *
     * @since 1.0
     */
   
    public String encodeOMObject( OMObject fOMObject )
        throws  CodecEncodeException
    {
    	// System.out.println( "list1Codec.encode" );
	
        if ( fOMObject instanceof OMApplication )
        {
            OMApplication tApplication = (OMApplication) fOMObject;
            
            if ( tApplication.getElementAt( 0 ) instanceof OMSymbol )
            {
                OMSymbol tOMSymbol = (OMSymbol) tApplication.getElementAt( 0 );
	    
                switch( getIndex( tOMSymbol.getName() ) )
                {
                    case 0 : return encodeList( fOMObject );
                    case 1 : return encodeMap( fOMObject );
                    case 2 : return encodeSuchThat( fOMObject );
                }
            }
        }
    
        throw new CodecEncodeException( 
            "Unable to encode: " + fOMObject );
    }


    /**
     * Decodes the given Mathematica-syntax to a OMObject. <p>
     *
     * @since 1.0
     */
  
    public OMObject decodeOMObject( String fMathematicaSyntax )
        throws  CodecDecodeException
    {
	// System.out.println( "list1Codec.decodeOMObject" );
        
        String tOperator = 
	    fMathematicaSyntax.substring( 
                0, fMathematicaSyntax.indexOf( '[' ) );

	String tArguments =
	    fMathematicaSyntax.substring( fMathematicaSyntax.indexOf( '[' ) );

	if ( tOperator != null )
	{
	    tOperator = tOperator.trim();

	    switch( getMathIndex( tOperator ) )
	    {		
	        case 0  : return decodeList( tArguments );
                default : break;
	    }
	}
       
        throw new CodecDecodeException( 
            "Unable to decode: " + fMathematicaSyntax );
    }
  
  
    /**
     * Gets the index for the name. <p>
     *
     * @since 1.0
     */
   
    private int getIndex( String fName )
    {
	// System.out.println( "list1Codec.getIndex" );
	
	for( int i=0; i<sNames.length; i++ )
	{
	    if ( fName.equals( sNames[i] ) )
		return i;
	}
    
	return -1;
    }
 

    /**
     * Gets the index for the Mathematica operator. <p>
     *
     * @since 1.0
     */
   
    private int getMathIndex( String fOperator )
    {
	// System.out.println( "integer1Codec.getMathIndex" );
	
	for( int i=0; i<sNames.length; i++ )
	{
	    if ( fOperator.equals( sMathNames[i] ) )
		return i;
	}
    
	return -1;
    }

    
    /**
     * Encodes the list1.list. <p>
     *
     * @since 1.0
     */
   
    public String encodeList( OMObject fOMObject )
        throws  CodecEncodeException
    {
	    // System.out.println( "list1Codec.encodeList" );
    
    	StringBuffer tResult = new StringBuffer();
	    OMApplication tApplication = (OMApplication) fOMObject;

        int tIndex = 1;
        
	tResult.append( "List[" );
	
        while( tApplication.getElementAt( tIndex ) != null )
        {
            tResult.append( 
                mParent.encodeOMObject( tApplication.getElementAt( tIndex ) ) );
            
            tIndex++;
            
            if ( tApplication.getElementAt( tIndex ) != null )
                tResult.append( "," );
    	}

	tResult.append( "]" );
    	return tResult.toString();    
    }


    /**
     * Encodes the list1.map. <p>
     *
     * @since 1.0
     */
   
    public String encodeMap( OMObject fOMObject )
        throws  CodecEncodeException
    {
    	// System.out.println( "list1Codec.encodeMap" );
     
        throw new CodecEncodeException( 
            "Unable to encode: " + fOMObject.toString() );
    }


    /**
     * Encodes the list1.such_that. <p>
     *
     * @since 1.0
     */
   
    public String encodeSuchThat( OMObject fOMObject )
        throws  CodecEncodeException
    {
    	// System.out.println( "list1Codec.encodeSuchThat" );
     
        throw new CodecEncodeException( 
            "Unable to encode: " + fOMObject.toString() );
    }
    
    
    
    /**
     * Decodes the Mathematica (List) expression. <p>
     *
     * @since 1.0
     */

    public OMObject decodeList( String fMathematicaSyntax )
        throws  CodecDecodeException
    {
    	// System.out.println( "integer1Codec.decodeQuotient" );

	OMApplication tOMApplication = new OMApplication();
           
    	tOMApplication.addElement( new OMSymbol( "list1", "list" ) );
	    
    	MathematicaCodec tParentCodec = (MathematicaCodec) mParent;
	Vector tVector = tParentCodec.parseArguments( fMathematicaSyntax );
        
    	int tSize = tVector.size();
        
        int i = 0;
        
        /**
         * A list can be empty!
         */
            
        while ( i < tSize )
        {
            OMObject tOMObject = 
		tParentCodec.decodeOMObject( 
		    (String) tVector.elementAt( i ) );
                
 
            tOMApplication.addElement( tOMObject );
            i++;            
        }
	
	return tOMApplication;
    }
}

