/*
 *  $Id: nums1Codec.java,v 1.5 2003/01/17 09:46:20 mriem Exp $
 *
 *  Copyright (c) 1999-2002, RIACA, Technische Universiteit Eindhoven (TU/e).
 *  All Rights Reserved.
 *
 *  The contents of this file are subject to the RIACA Public License
 *  Version 1.0 (the "License"); you may not use this file except in
 *  compliance with the License. A copy of the License is available at
 *  http://www.riaca.win.tue.nl/
 */


package mathematicaCodec;


import nl.tue.win.riaca.openmath.codec.*;
import nl.tue.win.riaca.openmath.lang.*;
import java.util.*;


/**
 * A CD-codec that handles the OpenMath 'nums1' CD. <p>
 *
 * @author Manfred Riem (mriem@win.tue.nl)
 * @version 1.1
 */

public class nums1Codec
    extends CDCodec {

    /**
     * Stores the names in this codec. <p>
     *
     * @since 1.0
     */
    
    public String[] sNames = 
    {
        "based_integer",
        "e",
        "gamma",
        "i",
        "infinity",
        "NaN",
        "pi",
        "rational"
    };
    
    
    /**
     * Stores the Mathematica names in this codec. <p>
     *
     * @since 1.0
     */
    
    public String[] sMathNames = 
    {
        "E",
        "EulerGamma",
        "I",
        "Infinity",
        "Indeterminate",
	"Pi",
        "Rational"
    };
    
    
    /**
     * Constructor. <p>
     *
     * @since 1.0
     */

    public nums1Codec()
    {
	super();
        
	// System.out.println( "nums1Codec.<init>" );
    }
    
    
    /**
     * Constructor. <p>
     *
     * @since 1.0
     */
    
    public nums1Codec( Codec fParent )
    {
	super();
	    
	mParent = fParent;
	
	// System.out.println( "nums1Codec.<init>" );
    }
    
    
    /**
     * Encodes the given OMObject to Mathematica-syntax. <p>
     *
     * @since 1.0
     */
    
    public String encodeOMObject( OMObject fOMObject )
        throws CodecEncodeException
    {
	// System.out.println( "nums1Codec.encode" );
	    
	if ( fOMObject instanceof OMApplication ) {

	    OMApplication tApplication = (OMApplication) fOMObject;
	    OMSymbol tOMSymbol = 
		(OMSymbol) tApplication.firstElement();
	    
            switch( getIndex( tOMSymbol.getName() ) )
		{
                case 7 : return encodeRational( fOMObject );
		}
	}
	
	if ( fOMObject instanceof OMSymbol ) {
	    OMSymbol tOMSymbol = (OMSymbol) fOMObject;
            
            switch( getIndex( tOMSymbol.getName() ) )
		{
                case 0 : break;
                case 1 : return encodeE( fOMObject );
                case 2 : return encodeGamma( fOMObject );
                case 3 : return encodeI( fOMObject );
                case 4 : return encodeInfinity( fOMObject );
                case 5 : return encodeNaN( fOMObject );
                case 6 : return encodePi( fOMObject );
		}
	}
	
        throw new CodecEncodeException( 
            "Unable to encode : " + fOMObject );
    }
    
    
    /**
     * Decodes the given Mathematica-syntax to a OMObject. <p>
     *
     * @since 1.0
     */
    
    public OMObject decodeOMObject( String fMathematicaSyntax )
        throws CodecDecodeException
    {
	// System.out.println( "nums1Codec.decode" );
        
        try
        {
            String tOperator = 
                fMathematicaSyntax.substring( 
                    0, fMathematicaSyntax.indexOf( '[' ) );
            
            String tArguments =
                fMathematicaSyntax.substring( 
                    fMathematicaSyntax.indexOf( '[' ) );
            
            if ( tOperator != null )
            {
                tOperator = tOperator.trim();
                
                switch( getMathIndex( tOperator ) )
                {
                    case 6  : return decodeRational( tArguments );
                    default : break;
                }
            }
        }
        catch( Exception tException ) {}
        
        String tSymbol = fMathematicaSyntax;
        tSymbol = tSymbol.trim();
        
        switch( getMathIndex( tSymbol ) )
        {
            case 0 : return decodeE( tSymbol );
            case 1 : return decodeEulerGamma( tSymbol );
            case 2 : return decodeI( tSymbol );
            case 3 : return decodeInfinity( tSymbol );
            case 4 : return decodeIndeterminate( tSymbol );
            case 5 : return decodePi( tSymbol );
        }
        
	throw new CodecDecodeException( "Unable to decode: " + fMathematicaSyntax );
    }
    
    
    /**
     * Gets the index for the name. <p>
     *
     * @since 1.0
     */
    
    private int getIndex( String fName )
    {
	// System.out.println( "nums1Codec.getIndex" );
	    
	for( int i=0; i<sNames.length; i++ )
	{
	    if ( fName.equals( sNames[i] ) )
		return i;
	}
        
	return -1;
    }
    
    
    /**
     * Gets the index for the Mathematica operator. <p>
     *
     * @since 1.0
     */
    
    private int getMathIndex( String fOperator )
    {
	// System.out.println( "nums1Codec.getMathIndex" );
	    
	for( int i=0; i<sMathNames.length; i++ )
    	{
	    if ( fOperator.equals( sMathNames[i] ) )
	    	return i;
	}
	
	return -1;
    }
    
    
    /**
     * Encodes nums.e symbol. <p>
     *
     * @since 1.0
     */
    
    public String encodeE( OMObject fOMObject )
        throws CodecEncodeException
    {
	// System.out.println( "nums1Codec.encodePi" );
	
    	return "E";
    }


    /**
     * Encodes nums.gamma symbol. <p>
     *
     * @since 1.0
     */
    
    public String encodeGamma( OMObject fOMObject )
        throws CodecEncodeException
    {
	// System.out.println( "nums1Codec.encodeGamma" );
        
	return "EulerGamma";
    }
    
    
    /**
     * Encodes nums.i symbol. <p>
     *
     * @since 1.0
     */
    
    public String encodeI( OMObject fOMObject )
        throws CodecEncodeException
    {
	// System.out.println( "nums1Codec.encodeI" );
        
	return "I";
    }
    
    
    /**
     * Encodes nums.infinity symbol. <p>
     *
     * @since 1.0
     */
    
    public String encodeInfinity( OMObject fOMObject )
        throws CodecEncodeException
    {
	// System.out.println( "nums1Codec.encodeInfinity" );
        
	return "Infinity";
    }
    
    
    /**
     * Encodes nums.NaN symbol. <p>
     *
     * @since 1.0
     */
    
    public String encodeNaN( OMObject fOMObject )
        throws  CodecEncodeException
    {
	// System.out.println( "nums1Codec.encodeNaN" );
        
	return "Indeterminate";
    }
    
    
    /**
     * Encodes nums.pi symbol. <p>
     *
     * @since 1.0
     */
    
    public String encodePi( OMObject fOMObject )
        throws  CodecEncodeException
    {
	// System.out.println( "nums1Codec.encodePi" );
        
	return "Pi";
    }
    
    
    /**
     * Encode nums1.rational <p>
     *
     * @param the OMObject to encode. 
     * @since 1.0
     */
    
    public String encodeRational( OMObject fOMObject )
        throws  CodecEncodeException {
	
	// System.out.println( "nums1Codec.encodeRational" );
        
	StringBuffer  tResult      = new StringBuffer();
	OMApplication tApplication = (OMApplication) fOMObject;
        
	tResult.append( "Rational[" );
	
	if ( tApplication.getElementAt( 1 ) != null ) {
	    tResult.append(
	        mParent.encodeOMObject( tApplication.getElementAt( 1 ) ) );
	}
	else {
	    throw 
		new CodecEncodeException( "Unable to encode : " + fOMObject );
	}
        
	tResult.append( "," );
        
        if ( tApplication.getElementAt( 2 ) != null ) {
	    tResult.append(
	        mParent.encodeOMObject( tApplication.getElementAt( 2 ) ) );
	}
	else {
	    throw 
		new CodecEncodeException( "Unable to encode : " + fOMObject );
	}
        
	tResult.append( "]" );
	
	return tResult.toString();
    }
    
    
    /**
     * Decodes 'E' in Mathematica. <p>
     *
     * @since 1.0
     */
    
    public OMObject decodeE( String fMathematicaSyntax )
        throws CodecDecodeException
    {
	// System.out.println( "nums1Codec.decodeE" );
        
	return new OMSymbol( "nums1", "e" );
    }
    
    
    /**
     * Decodes 'EulerGamma' in Mathematica. <p>
     *
     * @since 1.0
     */
    
    public OMObject decodeEulerGamma( String fMathematicaSyntax )
        throws CodecDecodeException
    {
	// System.out.println( "nums1Codec.decodeEulerGamma" );
        
	return new OMSymbol( "nums1", "gamma" );
    }
    
    
    /**
     * Decodes 'I' in Mathematica. <p>
     *
     * @since 1.0
     */
    
    public OMObject decodeI( String fMathematicaSyntax )
        throws CodecDecodeException
    {
	// System.out.println( "nums1Codec.decodeI" );
	
	return new OMSymbol( "nums1", "i" );
    }
    
    
    /**
     * Decodes 'Infinity' in Mathematica. <p>
     *
     * @since 1.0
     */
    
    public OMObject decodeInfinity( String fMathematicaSyntax )
        throws CodecDecodeException
    {
	// System.out.println( "nums1Codec.decodeInfinity" );
        
	return new OMSymbol( "nums1", "infinity" );
    }
    
    
    /**
     * Decodes 'Indeterminate' in Mathematica. <p>
     *
     * @since 1.0
     */
    
    public OMObject decodeIndeterminate( String fMathematicaSyntax )
        throws CodecDecodeException
    {
	// System.out.println( "nums1Codec.decodeIndeterminate" );
        
	return new OMSymbol( "nums1", "NaN" );
    }
    
    
    /**
     * Decodes 'Pi' in Mathematica. <p>
     *
     * @since 1.0
     */
    
    public OMObject decodePi( String fMathematicaSyntax )
        throws CodecDecodeException
    {
	// System.out.println( "nums1Codec.decodePi" );
        
	return new OMSymbol( "nums1", "pi" );
    }
    
    
    /**
     * Decodes 'Rational' in Mathematica. <p>
     *
     * @since 1.0
     */
    
    public OMObject decodeRational( String fMathematicaSyntax )
        throws CodecDecodeException
    {
	// System.out.println( "nums1Codec.decodeRational" );
        
	OMApplication tOMApplication = new OMApplication();
	tOMApplication.addElement( new OMSymbol( "nums1", "rational" ) );
	    
	MathematicaCodec tParentCodec = (MathematicaCodec) mParent;
	Vector tVector = tParentCodec.parseArguments( fMathematicaSyntax );
        
	if ( tVector.size() == 2 )
	{
	    OMObject tOMObject = 
		tParentCodec.decodeOMObject( 
                    (String) tVector.elementAt( 0 ) );
            
	    tOMApplication.addElement( tOMObject );
            
	    tOMObject = tParentCodec.decodeOMObject(
		(String) tVector.elementAt( 1 ) );		
	        
	    tOMApplication.addElement( tOMObject );
	}
	else
            throw new CodecDecodeException( 
	        "Unable to decode: " + fMathematicaSyntax );
	
	return tOMApplication;
    }
}
