/*
 * $Id: relation1Codec.java,v 1.4 2002/11/20 23:51:51 mriem Exp $
 *
 * Copyright (c) 1999-2002, RIACA, Technische Universiteit Eindhoven (TU/e).
 * All Rights Reserved.
 *
 * The contents of this file are subject to the RIACA Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.riaca.win.tue.nl/
 */


package mathematicaCodec;

import nl.tue.win.riaca.openmath.codec.*;
import nl.tue.win.riaca.openmath.lang.*;
import java.util.*;


/**
 * A CD-codec that handles the OpenMath 'relation1' CD. <p>
 *
 * @author Manfred Riem (mriem@win.tue.nl)
 * @version 1.0
 */

public class relation1Codec
    extends CDCodec
{
    /**
     * Stores the names in this codec. <p>
     *
     * @since 1.0
     */
   
    public String[] sNames = 
    {
    	"approx",
    	"eq",
    	"geq",
    	"gt",
    	"leq",
    	"lt",
    	"neq"
    };


    /**
     * Stores the Mathematica names in this codec. <p>
     *
     * @since 1.0
     */
   
    public String[] sMathNames = 
    {
    	"Equal",
    	"GreaterEqual",
    	"Greater",
    	"LessEqual",
    	"Less",
    	"Unequal"
    };

    
    /**
     * Constructor. <p>
     *
     * @since 1.0
     */

    public relation1Codec()
    {
	super();

	// System.out.println( "relation1Codec.<init>" );
    }

 
    /**
     * Constructor. <p>
     *
     * @since 1.0
     */
   
    public relation1Codec( Codec fParent )
    {
    	super();
	
    	mParent = fParent;
	
	// System.out.println( "relation1Codec.<init>" );
    }
  
  
    /**
     * Encodes the given OMObject to Mathematica-syntax. <p>
     *
     * @since 1.0
     */
   
    public String encodeOMObject( OMObject fOMObject )
        throws  CodecEncodeException
    {
    	// System.out.println( "relation1Codec.encode" );
	
        if ( fOMObject instanceof OMApplication )
        {
            OMApplication tApplication = (OMApplication) fOMObject;
            OMObject      tOMObject    = tApplication.getElementAt( 0 );
	
            if ( tOMObject instanceof OMSymbol )
            {
                OMSymbol tOMSymbol = (OMSymbol) tOMObject;
	    
                switch( getIndex( tOMSymbol.getName() ) )
                {
                    case 0  : return encodeApprox( fOMObject );
                    case 1  : return encodeEqual( fOMObject );
                    case 2  : return encodeGreaterEqual( fOMObject );
                    case 3  : return encodeGreaterThan( fOMObject );
                    case 4  : return encodeLessEqual( fOMObject );
                    case 5  : return encodeLessThan( fOMObject );
                    case 6  : return encodeNotEqual( fOMObject );
                }
            }
	}
	
        throw new CodecEncodeException( "Unable to encode: " + fOMObject.toString() );
    }


    /**
     * Decodes the given Mathematica-syntax to a OMObject. <p>
     *
     * @since 1.0
     */
  
    public OMObject decodeOMObject( String fMathematicaSyntax )
        throws  CodecDecodeException
    {
    	// System.out.println( "relation1Codec.decode" );
       
        try
        {
            String tOperator = 
                fMathematicaSyntax.substring( 
                    0, fMathematicaSyntax.indexOf( '[' ) );

            String tArguments =
                fMathematicaSyntax.substring( 
                    fMathematicaSyntax.indexOf( '[' ) );

            if ( tOperator != null )
            {
                tOperator = tOperator.trim();

                switch( getMathIndex( tOperator ) )
                {
                    case 0  : return decodeEqual( tArguments );
                    case 1  : return decodeGreaterEqual( tArguments );
                    case 2  : return decodeGreater( tArguments );
                    case 3  : return decodeLessEqual( tArguments );
                    case 4  : return decodeLessThan( tArguments );
                    case 5  : return decodeUnequal( tArguments );
                    default : break;
                }
            }
        }
        catch( Exception tException ) {}
    
        throw new CodecDecodeException( "Unable to decode: " + fMathematicaSyntax );
    }
  
  
    /**
     * Gets the index for the name. <p>
     *
     * @since 1.0
     */
   
    private int getIndex( String fName )
    {
    	// System.out.println( "relation1Codec.getIndex" );
	    
	for( int i=0; i<sNames.length; i++ )
	{
	    if ( fName.equals( sNames[i] ) )
		return i;
    	}
	
	return -1;
    }


    /**
     * Gets the index for the Mathematica operator. <p>
     *
     * @since 1.0
     */
   
    private int getMathIndex( String fOperator )
    {
    	// System.out.println( "relation1Codec.getMathIndex" );
	
	for( int i=0; i<sNames.length; i++ )
    	{
	    if ( fOperator.equals( sMathNames[i] ) )
		return i;
    	}
	
	return -1;
    }
    
    
    /**
     * Encodes the relation1.approx. <p>
     *
     * @since 1.0
     */
   
    public String encodeApprox( OMObject fOMObject )
        throws  CodecEncodeException
    {
        // System.out.println( "relation1Codec.encodeApprox" );
        
        throw new CodecEncodeException( "Unable to encode:" + fOMObject );
    }


    /**
     * Encodes the relation1.equal. <p>
     *
     * @since 1.0
     */
   
    public String encodeEqual( OMObject fOMObject )
        throws  CodecEncodeException
    {
    	// System.out.println( "relation1Codec.encodeEqual" );
	
	StringBuffer  tResult      = new StringBuffer();
    	OMApplication tApplication = (OMApplication) fOMObject;
	
	tResult.append( "Equal[" );
	
    	if ( tApplication.getElementAt( 1 ) != null )
	    tResult.append(
	        mParent.encodeOMObject( tApplication.getElementAt( 1 ) ) );
    	else
	    throw new CodecEncodeException( "Unable to encode:" + fOMObject );
	
    	tResult.append( "," );
	
        if ( tApplication.getElementAt( 2 ) != null )
	        tResult.append(
		         mParent.encodeOMObject( tApplication.getElementAt( 2 ) ) );
        else
	    throw new CodecEncodeException( "Unable to encode:" + fOMObject );
	
    	tResult.append( "]" );
	return tResult.toString();
    }


    /**
     * Encodes the relation1.geq. <p>
     *
     * @since 1.0
     */
   
    public String encodeGreaterEqual( OMObject fOMObject )
        throws  CodecEncodeException
    {
    	// System.out.println( "relation1Codec.encodeGreaterEqual" );
     
	StringBuffer  tResult      = new StringBuffer();
    	OMApplication tApplication = (OMApplication) fOMObject;
	
	tResult.append( "GreaterEqual[" );
	
    	if ( tApplication.getElementAt( 1 ) != null )
	    tResult.append(
	        mParent.encodeOMObject( tApplication.getElementAt( 1 ) ) );
    	else
	    throw new CodecEncodeException( "Unable to encode:" + fOMObject );
	
    	tResult.append( "," );
	
        if ( tApplication.getElementAt( 2 ) != null )
	    tResult.append(
	        mParent.encodeOMObject( tApplication.getElementAt( 2 ) ) );
    	else
	    throw new CodecEncodeException( "Unable to encode:" + fOMObject );
	
    	tResult.append( "]" );
	return tResult.toString();
    }
    
    
    /**
     * Encodes the relation1.gt. <p>
     *
     * @since 1.0
     */
    
    public String encodeGreaterThan( OMObject fOMObject )
        throws  CodecEncodeException
    {
	// System.out.println( "relation1Codec.encodeGreaterThan" );
     
    	StringBuffer  tResult      = new StringBuffer();
	OMApplication tApplication = (OMApplication) fOMObject;
    
    	tResult.append( "Greater[" );
	
	if ( tApplication.getElementAt( 1 ) != null )
	    tResult.append(
                mParent.encodeOMObject( tApplication.getElementAt( 1 ) ) );
    	else
	    throw new CodecEncodeException( "Unable to encode:" + fOMObject );
	
    	tResult.append( "," );
	
        if ( tApplication.getElementAt( 2 ) != null )
	    tResult.append(
	        mParent.encodeOMObject( tApplication.getElementAt( 2 ) ) );
    	else
            throw new CodecEncodeException( "Invalid format: " + fOMObject.toString() );
	
    	tResult.append( "]" );
	return tResult.toString();
    }
    
    
    /**
     * Encodes the relation1.leq. <p>
     *
     * @since 1.0
     */
    
    public String encodeLessEqual( OMObject fOMObject )
        throws  CodecEncodeException
    {
    	// System.out.println( "relation1Codec.encodeLessEqual" );
	
	StringBuffer  tResult      = new StringBuffer();
    	OMApplication tApplication = (OMApplication) fOMObject;
	
	tResult.append( "LessEqual[" );
	
    	if ( tApplication.getElementAt( 1 ) != null )
	    tResult.append(
	        mParent.encodeOMObject( tApplication.getElementAt( 1 ) ) );
    	else
	    throw new CodecEncodeException( "Unable to encode:" + fOMObject );
	
    	tResult.append( "," );
	
        if ( tApplication.getElementAt( 2 ) != null )
	    tResult.append(
	        mParent.encodeOMObject( tApplication.getElementAt( 2 ) ) );
	else
	    throw new CodecEncodeException( "Unable to encode:" + fOMObject );
	
    	tResult.append( "]" );  
	return tResult.toString();
    }
    
    
    /**Cross[List[a,b,c],List[x,y,z]
     * Encodes the relation1.lt. <p>
     *
     * @since 1.0
     */
    
    public String encodeLessThan( OMObject fOMObject )
        throws  CodecEncodeException
    {
	// System.out.println( "relation1Codec.encodeLessThan" );
	
    	StringBuffer  tResult      = new StringBuffer();
	OMApplication tApplication = (OMApplication) fOMObject;
	
    	tResult.append( "Less[" );
	
	if ( tApplication.getElementAt( 1 ) != null )
	    tResult.append(
                mParent.encodeOMObject( tApplication.getElementAt( 1 ) ) );
    	else
	    throw new CodecEncodeException( "Unable to encode:" + fOMObject );
	
    	tResult.append( "," );
	
        if ( tApplication.getElementAt( 2 ) != null )
	    tResult.append(
	        mParent.encodeOMObject( tApplication.getElementAt( 2 ) ) );
    	else
	    throw new CodecEncodeException( "Unable to encode:" + fOMObject );
	
    	tResult.append( "]" );
	return tResult.toString();
    }
    
    
    /**
     * Encodes the relation1.neq. <p>
     *
     * @since 1.0
     */
    
    public String encodeNotEqual( OMObject fOMObject )
        throws  CodecEncodeException
    {
    	// System.out.println( "relation1Codec.encodeNotEqual" );
	
	StringBuffer  tResult      = new StringBuffer();
    	OMApplication tApplication = (OMApplication) fOMObject;
	
	tResult.append( "Unequal[" );
	
    	if ( tApplication.getElementAt( 1 ) != null )
	    tResult.append(
	        mParent.encodeOMObject( tApplication.getElementAt( 1 ) ) );
    	else
	    throw new CodecEncodeException( "Unable to encode:" + fOMObject );
	
	tResult.append( "," );
        
        if ( tApplication.getElementAt( 2 ) != null )
	    tResult.append(
	        mParent.encodeOMObject( tApplication.getElementAt( 2 ) ) );
	else
	    throw new CodecEncodeException( "Unable to encode:" + fOMObject );
        
	tResult.append( "]" );
	return tResult.toString();
    }
    
    
    /**
     * Decodes the Mathematica (Equal) expression. <p>
     *
     * @since 1.0
     */
    
    public OMObject decodeEqual( String fMathematicaSyntax )
        throws CodecDecodeException
    {
	// System.out.println( "relation1Codec.decodeEqual" );
        
	OMApplication tOMApplication = new OMApplication();
	tOMApplication.addElement( new OMSymbol( "relation1", "eq" ) );
	
	MathematicaCodec tParentCodec = (MathematicaCodec) mParent;
	Vector tVector = tParentCodec.parseArguments( fMathematicaSyntax );
        
	if ( tVector.size() == 2 )
	{
	    OMObject tOMObject = 
		tParentCodec.decodeOMObject( 
                    (String) tVector.elementAt( 0 ) );
	    
	    tOMApplication.addElement( tOMObject );
	    
    	    tOMObject = tParentCodec.decodeOMObject(
		(String) tVector.elementAt( 1 ) );		
	    
	    tOMApplication.addElement( tOMObject );
    	}
	else
            throw new CodecDecodeException( "Unable to decode: " + fMathematicaSyntax );
	
    	return tOMApplication;  
    }
    
    
    /**
     * Decodes the Mathematica (GreaterEqual) expression. <p>
     *
     * @since 1.0
     */

    public OMObject decodeGreaterEqual( String fMathematicaSyntax )
        throws  CodecDecodeException
    {
	// System.out.println( "relation1Codec.decodeGreaterEqual" );
	
	OMApplication tOMApplication = new OMApplication();
    	tOMApplication.addElement( new OMSymbol( "relation1", "geq" ) );
	
	MathematicaCodec tParentCodec = (MathematicaCodec) mParent;
    	Vector tVector = tParentCodec.parseArguments( fMathematicaSyntax );
	
	if ( tVector.size() == 2 )
    	{
	    OMObject tOMObject = 
		tParentCodec.decodeOMObject( 
                    (String) tVector.elementAt( 0 ) );
            
    	    tOMApplication.addElement( tOMObject );
	    
	    tOMObject = tParentCodec.decodeOMObject(
	        (String) tVector.elementAt( 1 ) );		
	    
    	    tOMApplication.addElement( tOMObject );
	}
    	else
            throw new CodecDecodeException( "Unable to decode: " + fMathematicaSyntax );
	
    	return tOMApplication;
    }
    
    
    /**
     * Decodes the Mathematica (Greater) expression. <p>
     *
     * @since 1.0
     */
    
    public OMObject decodeGreater( String fMathematicaSyntax )
        throws  CodecDecodeException
    {
	// System.out.println( "relation1Codec.decodeGreater" );
	
    	OMApplication tOMApplication = new OMApplication();
	tOMApplication.addElement( new OMSymbol( "relation1", "gt" ) );
	
    	MathematicaCodec tParentCodec = (MathematicaCodec) mParent;
	Vector tVector = tParentCodec.parseArguments( fMathematicaSyntax );
	
    	if ( tVector.size() == 2 )
	{
	    OMObject tOMObject = 
		tParentCodec.decodeOMObject( 
		    (String) tVector.elementAt( 0 ) );
	    
	    tOMApplication.addElement( tOMObject );
	    
    	    tOMObject = tParentCodec.decodeOMObject(
	        (String) tVector.elementAt( 1 ) );		
	    
	    tOMApplication.addElement( tOMObject );
    	}
	else
            throw new CodecDecodeException( "Unable to decode: " + fMathematicaSyntax );
	
    	return tOMApplication;
    }
    
    
    /**
     * Decodes the Mathematica (LessEqual) expression. <p>
     *
     * @since 1.0
     */
    
    public OMObject decodeLessEqual( String fMathematicaSyntax )
        throws  CodecDecodeException
    {
	// System.out.println( "relation1Codec.decodeLessEqual" );
	
    	OMApplication tOMApplication = new OMApplication();
	tOMApplication.addElement( new OMSymbol( "relation1", "leq" ) );
	
    	MathematicaCodec tParentCodec = (MathematicaCodec) mParent;
	Vector tVector = tParentCodec.parseArguments( fMathematicaSyntax );
	
    	if ( tVector.size() == 2 )
	{
	    OMObject tOMObject = 
		tParentCodec.decodeOMObject( 
                    (String) tVector.elementAt( 0 ) );
	    
	    tOMApplication.addElement( tOMObject );
	    
    	    tOMObject = tParentCodec.decodeOMObject(
		(String) tVector.elementAt( 1 ) );		
	    
	    tOMApplication.addElement( tOMObject );
    	}
	else
            throw new CodecDecodeException( "Unable to decode: " + fMathematicaSyntax );
	
    	return tOMApplication;
    }
    
    
    /**
     * Decodes the Mathematica (LessThan) expression. <p>
     *
     * @since 1.0
     */
    
    public OMObject decodeLessThan( String fMathematicaSyntax )
        throws  CodecDecodeException
    {
	// System.out.println( "relation1Codec.decodeLessThan" );
	
    	OMApplication tOMApplication = new OMApplication();
	tOMApplication.addElement( new OMSymbol( "relation1", "lt" ) );
	
    	MathematicaCodec tParentCodec = (MathematicaCodec) mParent;
	Vector tVector = tParentCodec.parseArguments( fMathematicaSyntax );
	
    	if ( tVector.size() == 2 )
	{
	    OMObject tOMObject = 
		tParentCodec.decodeOMObject( 
                    (String) tVector.elementAt( 0 ) );
	    
	    tOMApplication.addElement( tOMObject );
	    
	    tOMObject = tParentCodec.decodeOMObject(
		(String) tVector.elementAt( 1 ) );		
	    
    	    tOMApplication.addElement( tOMObject );
	}
    	else
            throw new CodecDecodeException( "Unable to decode: " + fMathematicaSyntax );
	
    	return tOMApplication;
    }
    
    
    /**
     * Decodes the Mathematica (Unequal) expression. <p>
     *
     * @since 1.0
     */
    
    public OMObject decodeUnequal( String fMathematicaSyntax )
        throws  CodecDecodeException
    {
    	// System.out.println( "relation1Codec.decodeUnequal" );
	
	OMApplication tOMApplication = new OMApplication();    
    	tOMApplication.addElement( new OMSymbol( "relation1", "neq" ) );
	
	MathematicaCodec tParentCodec = (MathematicaCodec) mParent;
    	Vector tVector = tParentCodec.parseArguments( fMathematicaSyntax );
	
	if ( tVector.size() == 2 )
    	{
	    OMObject tOMObject = 
		tParentCodec.decodeOMObject( 
                    (String) tVector.elementAt( 0 ) );
	    
    	    tOMApplication.addElement( tOMObject );
	    
	    tOMObject = tParentCodec.decodeOMObject(
	        (String) tVector.elementAt( 1 ) );		
	    
    	    tOMApplication.addElement( tOMObject );
	}
    	else
            throw new CodecDecodeException( "Unable to decode: " + fMathematicaSyntax );
	
    	return tOMApplication;
    }
}

